from django import forms


def stylised(fn, *args, **kwargs):
    def wrapper():
        field = fn(*args, **kwargs)
        return field

    return wrapper


class _Field(forms.Field):
    def __init__(self, *args, **kwargs):
        super(_Field, self).__init__(*args, **kwargs)
        self.widget.attrs.setdefault('style', '')
        self.widget.attrs['style'] += ' display: none;'


class IntegerField(forms.IntegerField, _Field):
    @classmethod
    def of_range(cls, _min, _max): return cls(min_value=_min, max_value=_max, widget=forms.NumberInput(attrs={'type': 'range'}))

    @classmethod
    def of_input(cls, _min, _max): return cls(min_value=_min, max_value=_max)


class BooleanField(forms.BooleanField, _Field):
    pass


class ChoiceField(forms.ChoiceField, _Field):
    @classmethod
    def of_range(cls, _min, _max, *addition):
        if _max == 255:
            _max = 254
            addition = ((255, 'MASK'), *addition)
        return cls(choices=(('', ''), *((i, i) for i in range(_min, _max+1)), *addition))

    @classmethod
    def of_boolean(cls): return cls(choices=(('', ''), ('true', 'On'), ('false', 'Off'),))

    @classmethod
    def of_fade_time(cls): return cls(choices=(
        ('', ''), (0, 0.0), (1, 0.7), (2, 1.0), (3, 1.4), (4, 2.0), (5, 2.8), (6, 4.0), (7, 5.7), (8, 8.0), (9, 11.3),
        (10, 16.0), (11, 22.6), (12, 32.0), (13, 45.3), (14, 64.0), (15, 90.5),
    ))

    @classmethod
    def of_fade_rate(cls): return cls(choices=(
        ('', ''), (1, 358), (2, 253), (3, 179), (4, 127), (5, 89.4), (6, 63.3), (7, 44.7), (8, 31.6), (9, 22.4),
        (10, 15.8), (11, 11.2), (12, 7.9), (13, 5.6), (14, 4.0), (15, 2.8),
    ))

    @classmethod
    def of_dimming_curve(cls): return cls(choices=(('', ''), (0, 'Logarithmic'), (1, 'Linear'),))

    @classmethod
    def of_event_priority(cls): return cls.of_range(2, 5)

    @classmethod
    def of_operating_mode(cls): return cls(choices=(
        ('', ''), (0, 'Standard'), *((i, '0x' + hex(i)[2:].rjust(2, '0').upper()) for i in range(0x80, 0xff+1)))
    )

    @classmethod
    def of_event_scheme(cls): return cls(choices=(
        ('', ''),
        (0, 'Type/Index'),
        (1, 'Address/Type'),
        (2, 'Address/Index'),
        (3, 'Group/Type'),
        (4, 'IGroup/Type'),
    ))

    @classmethod
    def of_event_filter(cls): return cls(choices=(('', ''), *((i, bin(i)[2:].rjust(8, '0')) for i in range(0, 256))))

    @classmethod
    def of_instance_group(cls): return cls(choices=(('', ''), *((i, i) for i in range(0, 32)), (255, 'MASK')))

    @classmethod
    def _of_timer(cls, multiplier: (int, float), _min: int, _max: int, decimal=0, addition=()): return cls(choices=(
        ('', ''), *addition, *((i, round(i * multiplier, decimal)) for i in range(_min, _max + 1))
    ))

    @classmethod
    def of_dead_time(cls): return cls._of_timer(multiplier=0.05, _min=0, _max=255, decimal=2)

    @classmethod
    def of_report_time(cls): return cls._of_timer(multiplier=1, _min=0, _max=255)

    @classmethod
    def of_hold_time(cls): return cls._of_timer(multiplier=10, _min=0, _max=254)

    @classmethod
    def of_short_time(cls): return cls._of_timer(multiplier=0.02, _min=10, _max=255, decimal=2)

    @classmethod
    def of_double_time(cls): return cls._of_timer(multiplier=0.02, _min=10, _max=100, decimal=2)

    @classmethod
    def of_double_time_min(cls): return cls._of_timer(multiplier=0.02, _min=10, _max=100, decimal=2, addition=((0, 0),))

    @classmethod
    def of_repeat_time(cls): return cls._of_timer(multiplier=0.02, _min=5, _max=100, decimal=2)

    @classmethod
    def of_stuck_time(cls): return cls._of_timer(multiplier=1, _min=5, _max=255)
