import json

from asgiref.sync import async_to_sync
from channels.generic.websocket import WebsocketConsumer


class LineConsumer(WebsocketConsumer):
    channel_layer_alias = ''

    def __init__(self, *args, **kwargs):
        super(LineConsumer, self).__init__(*args, **kwargs)
        self.box = None
        self.module = None
        self.channel = None

    @property
    def room_name(self): return '%s-%s-%s' % self.dest

    @property
    def dest(self) -> tuple: return self.box, self.module, self.channel

    def connect(self):
        self.box = self.scope['url_route']['kwargs']['box']
        self.module = self.scope['url_route']['kwargs']['module']
        self.channel = self.scope['url_route']['kwargs']['channel']

        # Join room group
        async_to_sync(self.channel_layer.group_add)(self.room_name, self.channel_name)
        self.accept()
        self.channel_layer.get_status(*self.dest, self.channel_name)

    def disconnect(self, close_code):
        # Leave room group
        async_to_sync(self.channel_layer.group_discard)(
            self.room_name,
            self.channel_name
        )

    # Receive message from WebSocket
    def receive(self, text_data=None, bytes_data=None):
        raise NotImplementedError()

    # Receive message from room group
    def chat_message(self, event):
        message = event['message']

        # Send message to WebSocket
        self.send(text_data=json.dumps(message))
