from asgiref.sync import async_to_sync

from reface.line.worker import Worker
from resh.domains import dali1, dali2
from resh.utils.settings import conf


class Probator(Worker):
    file = 'probation'
    marks = {}

    def _run_thread(self, data):
        dali1_addresses = data.get('dali1', '')
        dali2_addresses = data.get('dali2', '')
        count = data['count']
        res = {}

        if dali1_addresses == dali2_addresses == '':
            dali1_addresses = dali2_addresses = ','.join('D%s' % i for i in range(0, 64))

        _d1 = len(dali1_addresses.split(',')) if dali1_addresses else 0
        _d2 = len(dali2_addresses.split(',')) if dali2_addresses else 0

        if _d1 + _d2 > 0:
            d1_count = _d1 / (_d1 + _d2)
            d2_count = _d2 / (_d1 + _d2)

            self.send_progress(0.01)

            conf.interface = self.box

            if dali1_addresses:
                self._command = dali1.test_line(self.module, self.channel, count, dali1_addresses, '-a')
                self._command.set_step_handler(lambda prs: self.on_step(int(prs*d1_count)))
                res[1] = self._command.exec()
            if dali2_addresses and not self.stopped:
                self._command = dali2.test_line(self.module, self.channel, count, dali2_addresses, '-a')
                self._command.set_step_handler(lambda prs: self.on_step(int(d1_count*100) + int(prs*d2_count)))
                res[2] = self._command.exec()

            self.send_result(res)
        self.send_progress(100)

    def stop_thread(self):
        super(Probator, self).stop_thread()
        if self._command:
            self._command.set_break(None, None)

    def on_step(self, progress):
        self.send_progress(progress)

    def send_result(self, data):
        self.write(self.file, value=data)
        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'result': data,
                }
            }
        )

    def send_progress(self, progress):
        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'progress': progress,
                }
            }
        )

    def send_status(self, channel_name):
        async_to_sync(self.layer.send)(
            channel_name,
            {
                'type': 'chat_message',
                'message': {'result': self.read(self.file)}
            }
        )
