"""
Django settings for reface project.

Generated by 'django-admin startproject' using Django 3.1.2.

For more information on this file, see
https://docs.djangoproject.com/en/3.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.1/ref/settings/
"""
import logging.config
import os
import sys

from django.utils.translation import gettext_lazy as _

from reface import const

LANGUAGES = [
    ('en', _('English')),
    ('ru', _('Russian')),
]

LANGUAGE_SESSION_KEY = 'session_language_appname'
LANGUAGE_COOKIE_NAME = 'cookie_language_appname'


DEFAULT_LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {},
        'require_debug_true': {},
    },
    'formatters': {
        'default': {
            'format': u'[%(name)s]: %(levelname)-8s [%(asctime)s]  %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': logging.DEBUG,
            'formatter': 'default',
            'class': 'logging.StreamHandler',
            'stream': sys.stdout
        },
    },
    'root': {
        'handlers': ['console'],
        'level': logging.DEBUG,
    },
    'loggers': {
        'can': {'level': logging.INFO},
        'daphne.ws_protocol': {'level': logging.INFO},
        'asyncio': {'level': logging.INFO},
        'can.interfaces.socketcan.socketcan': {'level': logging.WARNING},
        'daphne.http_protocol': {'level': logging.INFO}
    }
}

logging.config.dictConfig(DEFAULT_LOGGING)

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = '/usr/local/rapida/reface'


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'hefr_^vxspfb(zxy312xlwgz)ghzatmd-wi8a4^u$)=r+p!#84'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'reface.apps.RefaceConfig',
    'channels',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'reface.urls'
LOGIN_URL = '/rapida/auth/login/'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

ASGI_APPLICATION = 'reface.asgi.application'

CHANNEL_LAYERS = {
    'default': {
        'BACKEND': 'channels.layers.InMemoryChannelLayer',
    },
    const.LINE_BUILDER: {
        'BACKEND': 'reface.line.builder.layer.BuilderLayer',
    },
    const.LINE_SCANNER: {
        'BACKEND': 'reface.line.scanner.layer.ScannerLayer',
    },
    const.ADDRESS_SCANNER: {
        'BACKEND': 'reface.line.address.layer.AddressLayer',
    },
    const.LINE_PROBATION: {
        'BACKEND': 'reface.line.probation.layer.ProbatorLayer',
    },
}


# Database
# https://docs.djangoproject.com/en/3.1/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        # 'NAME': os.path.join(BASE_DIR, 'reface.sqlite3'),
        'NAME': ':memory:',
    },
    'auth_db': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'auth_db.sqlite3'),
    },
}

DATABASE_ROUTERS = ['reface.db_routing.AuthRouter']


# Password validation
# https://docs.djangoproject.com/en/3.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    # {
    #     'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    # },
    # {
    #     'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    # },
    # {
    #     'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    # },
    # {
    #     'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    # },
]


# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.1/howto/static-files/

STATIC_URL = '/rapida/static/'

# STATIC_ROOT = os.path.join(BASE_DIR, 'static/')
# python3 manage.py collectstatic

STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "static"),
]

LOCALE_PATHS = [
    os.path.join(BASE_DIR, 'static/locales/')
]
