const COLUMN = 'column',
    COMMAND = 'command',
    COMMON = 'common',
    INSTANCES = 'instances';

const CONTROL_ROW_ID = 'control';

const SWAP_ADDRESSES = 'swap_short_addresses';
const SET_ADDRESSES = 'set_short_address';
const REMOVE_ADDRESS = 'remove_short_address';

var chatSocket;
var filters = {};

filters[DALI1] = [];
filters[DALI2] = [];
filters[COLUMN] = [];
filters[COMMAND] = [];

window.onbeforeunload = function (e) {
    if (chatSocket && chatSocket.readyState === chatSocket.OPEN) {
        chatSocket.onclose = null;
        chatSocket.close();
    }
};

$(document).ready(function () {
    $(document).on('click', '.panel-heading', function () {
        var $this = $(this);
        $this.parents('.filters .panel').find('.panel-body').slideToggle();
        $this.find('i').toggleClass('glyphicon-chevron-down').toggleClass('glyphicon-chevron-up');
    });
    $(document).on('click', '.filter-btn', function () {
        $(document).find('.filters').slideToggle();
    });
    $(document).on('click', '[name=event_filter]', showMultiSelectFilters);
    $(document).on('click', '[name=groups]', showMultiSelectGroups);
    $(document).on('click', '[name=scenes]', showMultiSelectScenes);
    $(document).on('click', '#devices-table th', function (e) {
        const tbody = $('#devices-table tbody');
        const direct = e.target.classList.contains('ascended') ? -1 : 1;

        $('#devices-table th').removeClass('ascended').removeClass('descended');

        tbody.append(
            tbody.children().detach().get().sort(function (a, b) {
                var aVal = $(a).find('.'+Object.values(e.target.classList).join('.'))[0];
                var bVal = $(b).find('.'+Object.values(e.target.classList).join('.'))[0];

                if (aVal.childElementCount > 0 && bVal.childElementCount > 0) {
                    aVal = getInputValue(aVal.children[0]);
                    bVal = getInputValue(bVal.children[0]);

                    if (e.target.classList.contains('col-address')) {
                        aVal = parseInt(aVal.replace('D', ''));
                        bVal = parseInt(bVal.replace('D', ''));
                    } else if (!isNaN(aVal - parseFloat(aVal)) && !isNaN(bVal - parseFloat(bVal))) {
                        aVal = parseFloat(aVal);
                        bVal = parseFloat(bVal);
                    }

                    return direct * (aVal > bVal ? 1 : aVal === bVal ? 0 : -1);
                }
                return 0;
            })
        );
        e.target.classList.add(direct === 1 ? 'ascended' : 'descended');

        updateStripped();
    });

    $('#devices-table').find('[name=identify], [name=discovery]').show();

    addEventListener('filter', function (e) {
        var selector;
        if (e.detail.type === COLUMN) {
            selector = '.' + e.detail.type + (e.detail.item === 'BROADCAST' ? '' : '.' + e.detail.item);
            $('#devices-table ').find(selector)[e.detail.action]();
        } else if (e.detail.type === COMMAND) {
            selector = '.' + e.detail.type + (e.detail.item === 'BROADCAST' ? '' : '.'+e.detail.item);
            $('#devices-table ').find(selector)[e.detail.action]();

        } else if (e.detail.type === DALI1 || e.detail.type === DALI2) {
            selector = 'tr.' + e.detail.type + (e.detail.item === 'BROADCAST' ? '' : '.'+e.detail.item);
            $('#devices-table ').find(selector)[e.detail.action]();
            updateStripped();
        }
    });

    const commands = GET(COMMAND, []);
    $('.commands input[type=checkbox]').each(function (i, chb) {
        if (commands.length === 0 || commands.indexOf(chb.value) !== -1) {
            $(chb).click();
        } else {
            setColumn(chb);
        }
    });

    const columns = GET(COLUMN, []);
    $('.columns input[type=checkbox]').each(function (i, chb) {
        if (columns.length === 0 || columns.indexOf(chb.value) !== -1) {
            $(chb).click();
        } else {
            setColumn(chb);
        }
    });

    var devices = {};
    devices[DALI1] = GET(DALI1, []);
    devices[DALI2] = GET(DALI2, []);

    if (devices[DALI1].length === 0 && devices[DALI2].length === 0) {
        $('.filters .addresses.'+DALI1+' input[type=checkbox]').click();
        $('.filters .addresses.'+DALI2+' input[type=checkbox]').click();
    } else {
        for (var dali in devices) {
            for (var addr of devices[dali]) {
                if (!addr) continue;
                $('.filters .addresses.'+dali+' #filter-' + addr.toUpperCase()).click();
            }
        }
    }

    createControl();

    function createConnection() {
        updateLoading();
        const wsUrl = 'ws://' + window.location.host + '/rapida/ws/scanner/' + room + '/';
        console.log('Connect to ' + wsUrl);
        chatSocket = new WebSocket(wsUrl);

        chatSocket.onopen = function() { console.log('Connected'); };
        chatSocket.onclose = createConnection;
        chatSocket.onmessage = function (e) {
            const data = JSON.parse(e.data);

            console.log(data);
            if (data.hasOwnProperty('remove') && data.hasOwnProperty('type')) {
                $('#devices .' + data.remove + '.' + data.type).remove();
            } else if (data.hasOwnProperty('address') && data.hasOwnProperty('type') && data.hasOwnProperty('data')) {
                updateRow(data['address'], data['type'], data['data'], data['index']);
                updateCounter(data['address'], data['type'])
            } else if (data.hasOwnProperty('progress')) {
                updateProgress(data['progress']);
            } else if (data.hasOwnProperty('status')) {
                if (data.hasOwnProperty('loading')) {
                    updateLoading(data['loading']);
                }
                if (data.status) {
                    const daliType = data['status']['type'],
                        address = data['status']['address'],
                        device = data['status']['device'];
                    updateCounter(address, daliType);
                    if (device.hasOwnProperty('instances')) {
                        for (var instance of device['instances']) {
                            updateRow(address, daliType, instance, instance['index']);
                        }
                        delete device['instances'];
                    }
                    updateRow(address, daliType, device);
                }
            }
        };
    }

    createConnection();
});

function updateCounter(address, daliType) {
    address = address.toString();
    $('#'+daliType+'-count-input').each((i, input) => {
        var devices = input.value.split(',');
        if (devices[0] === '') devices.shift();
        if (devices.indexOf(address) === -1) {
            devices.push(address);
            input.value = devices.join(',');
        }
    })
        .trigger('change');
}

function createControl(address, daliType) {
    const table = document.getElementById('devices-table');
    const row = table.tHead.insertRow();
    row.id = CONTROL_ROW_ID;
    row.innerHTML = table.tHead.children[1].innerHTML;

    $(row).find('.field-value-read-only').remove();
    $(row).find(' select, input').show();
    $(row).find(' select, input').attr('disabled', 'disabled');
    $(row).find('.col-address select').each((i, select) => {
        select.removeAttribute('disabled');
        select.value = address;
        select.addEventListener('change', onControlAddressSelect);
    });
    $(row).find('.col-index').each(function(_, td) {
        td.colSpan = 2;
        $(td).find('input').remove();
        const select = document.createElement('select');
        td.appendChild(select);

        const setOpt = (owner, prefix, value) => {
            const opt = document.createElement('option');
            opt.value = prefix+value;
            opt.appendChild(document.createTextNode(opt.value));
            owner.appendChild(opt);
        };

        setOpt(select, '', '');
        const commonGroup = document.createElement('optgroup');
        commonGroup.label = 'Broadcast';
        setOpt(commonGroup, '', 'IB');
        select.appendChild(commonGroup);

        const typesGroup = document.createElement('optgroup');
        typesGroup.label = 'Type';
        [1, 3, 4].forEach((value) => setOpt(typesGroup, 'IT', value));
        select.appendChild(typesGroup);

        const indexesGroup = document.createElement('optgroup');
        indexesGroup.label = 'Index';
        for (var i=0; i < 32; i++) setOpt(indexesGroup, 'IN', i);
        select.appendChild(indexesGroup);
        select.addEventListener('change', onControlIndexSelect);
    });
    $(row).find('.col-dali-type select').each((i, select) => {
        select.removeAttribute('disabled');
        select.value = daliType;
        select.addEventListener('change', onControlDaliSelect);
        select.dispatchEvent(new Event('change'));
    });
    $(row).find('.col-type').remove();

    return row;
}

function updateRow(address, daliType, data, index) {
    if (!address.toString().startsWith('D')) address = 'D' + address;
    var rows = $('#devices .' + address + '.' + daliType + (index || index === 0 ? '.index-'+index : ''));
    if (rows.length === 0) {
        const table = document.getElementById('devices-table');
        const row = table.tBodies[0].insertRow();
        if (daliType === DALI2)
            index = index || 0;

        row.className = [address, daliType, 'index-'+index, 'selectable'].join(' ').trim();
        row.innerHTML = table.tHead.children[1].innerHTML;

        rows.push(row);

        if (daliType === DALI2) {
            $(row).find('td.col-index').children()[0].value = index;
            if (index > 0) {
                $('#devices .' + address + '.' + daliType + '.index-0 td.col-common').children()
                    .each((_, input) => updateCell($(row), input.name, getInputValue(input)));
            }
        }

        $(row).find('.col-number').text($(table).find('tbody tr:visible').length);

        if (daliType !== DALI1) $(row).find('.command.col-dali').remove();
        else if (daliType !== DALI2) $(row).find('.command.col-dali2').remove();

        if ($('.' + daliType + ' #filter-' + address + '.selected').length === 0) row.style.display = 'none';
        $(row).find('.col-address select').each((_, select) => {
            select.value = address;
            select.setAttribute('hasValue', address);
            select.addEventListener('change', setShortAddress);
            $(select).find(' optgroup[id=groups-addresses]').each((_, optgroup) => {
                const opt = document.createElement('option');
                opt.value = '255';
                opt.appendChild(document.createTextNode('MASK'));
                optgroup.appendChild(opt);
                $(optgroup).parents(' select').each((_, select) => {
                    $(select).children().remove();
                    select.appendChild(optgroup);
                });
            });
        });

        $(row).find('.col-dali-type select').each((_, s) => s.value = daliType).addClass('field-value-read-only');
    }

    for (var key in data) {
        if (data.hasOwnProperty(key)) {
            if (data[key].constructor === Object) {
                if (key === 'types') {
                    updateCell(rows, 'type', Object.keys(data[key]));
                } else if (key === 'scenes') {
                    updateCell(rows, key, JSON.stringify(data[key]));
                }else if (data[key].hasOwnProperty('type')) {
                    updateCell(rows, key, data[key]['type']);
                } else if (data[key].hasOwnProperty('desc')) {
                    updateCell(rows, key, data[key]['desc']);
                } else {
                    for (var subKey in data[key]) {
                        if (data[key].hasOwnProperty(subKey))
                            updateCell(rows, subKey, data[key][subKey]);
                    }
                }
            } else if (key === 'number_of_instances') {
                rows.slice(data[key]).remove();
            } else {
                updateCell(rows, key, data[key]);
            }
        }
    }
}

function updateCell(rows, key, value) {
    rows.find('.' + key).each(function(i, td) {
        if (td.childElementCount === 0) {
            td.appendChild(document.createElement('input'));
        }
        $(td).find('[name=' + key + ']').each((i, input) => {
            setInputValue(input, value);
            if (input.classList.contains('field-value-read-only')) {
                input.style.width = (getInputValue(input).length + 1) * 9 + 'px';
            }
        }).show();
    });
}

function setInputValue(input, value) {
    if (input.type === 'checkbox') {
        input.checked = value;
    } else {
        input.value = value;
    }
}

function getInputValue(input) {
    if (input.type === 'checkbox') {
        return input.checked
    } else {
        return input.value;
    }
}

function updateProgress(value) {
    $('#progress').show();
    $('#progress .progress-bar').css('width', value+'%')
        .attr('aria-valuenow', value)[0]
        .textContent = value + '%';
}

function updateLoading(value) {
    if (value === 100) {
        $('#loading').hide();
        $('#loaded-content').show();
        updateStripped();
    } else {
        $('#loaded-content').hide();
        $('#loading').show();
        value = value || 0;
        $('#loading progress')[0].value = value;
        $('#loading p').text(value + '%');
    }
}

function setAddress(e, broadcast) { setFilter(e, broadcast); return false; }
function setColumn(e, broadcast) { setFilter(e, broadcast); return false; }
function setCommand(e, broadcast) { setFilter(e, broadcast); return false; }
function setColumnGroup(e) { setFilter(e, true, e.getAttribute('value')); return false;}
function setCommandGroup(e) { setFilter(e, true, e.getAttribute('value')); return false;}

function setFilter(cb, broadcast, group) {
    function apply(cb, node) {
        if (cb.checked) {
            if (filters[node.name].indexOf(node.value) === -1) {
                filters[node.name].push(node.value);
            }
        } else {
            const index = filters[node.name].indexOf(node.value);
            if (index > -1) filters[node.name].splice(index, 1);
        }
    }

    function update(cb) {
        var filterName, value, action;
        if (cb.tagName === 'INPUT') {
            filterName = cb.name;
            value = cb.value;
            action = cb.checked ? 'show' : 'hide';
        } else {
            filterName = cb.getAttribute('name');
            action = cb.hasAttribute('checked') ? 'show' : 'hide';
        }
        $(cb).parents('.panel').find('.panel-heading span.badge').text(filters[filterName].length);
        dispatchEvent(new CustomEvent('filter', {
            detail: {
                type: filterName,
                item: broadcast ? group || 'BROADCAST' : value,
                action: action
            }
        }));
    }

    if (group) {
        $(cb).parents('.panel-body').each(function (i, panel) {
            const stack = $(panel).find('tr.' + cb.id).find('input[type=checkbox]').each(function (i, node) {
                node.checked = !node.checked;
                apply(node, node);
            });

            if (stack && stack[0].checked) {
                cb.setAttribute('checked', true);
                stack.parents('label').addClass('selected');
            }
            else {
                cb.removeAttribute('checked');
                stack.parents('label').removeClass('selected');
            }
            update(cb);

            return false;
        });
    } else if (broadcast) {
        $(cb).find('input[type=checkbox]').each(function (i, cb) {
            cb.checked = !cb.checked;
            $(cb).parents('.panel-body').each(function (i, panel) {
                const stack = $(panel).find('input[type=checkbox]').each(function (i, node) {
                    if (node === cb || node.checked !== cb.checked) {
                        node.checked = cb.checked;
                        apply(cb, node);
                    }
                });

                if (cb.checked) stack.parents('label').addClass('selected');
                else stack.parents('label').removeClass('selected');

                return false;
            });

            update(cb);
            return false;
        });
    } else if (cb.value !== '') {

        if (cb.checked) cb.labels[0].classList.add('selected');
        else cb.labels[0].classList.remove('selected');

        apply(cb, cb);
        update(cb);
    }
}

function sendCommand(input, command, ...params) {
    const row = $(input).parents(' tr');
    const daliType = row.find('.col-dali-type').children()[0].value;
    const address = row.find('.col-address').children()[0].value;
    const instance = (row[0].id === CONTROL_ROW_ID ? '' : daliType === DALI2 ? 'IN' : '') +
        row.find('.col-index').children()[0].value;

    if (command === undefined) command = input.name;

    console.log(command, address, ...params);

    chatSocket.send(JSON.stringify({
        type: 'exec',
        message: {
            command: command,
            type: daliType,
            address: address,
            instance: instance,
            params: params
        }
    }));
}

function setShortAddress(e) {
    const select = e.target;
    const daliType = $(select).parents('tr').find('.col-dali-type').children()[0].value;
    const newAddress = select.value;
    select.value = select.getAttribute('hasValue');
    var command;

    if (newAddress === '255') {
        if (window.confirm('Удалить устройство с адресом ' + select.value.substr(1) + '?')) {
            command = REMOVE_ADDRESS;
        }
    } else if ($('#devices-table').find('tr.'+newAddress+'.'+daliType).length > 0) {
        if (window.confirm('Устройство с адресом ' + newAddress.substr(1) + ' существует!\nЗаменить?')) {
            command = SWAP_ADDRESSES;
        }
    } else {
        command = SET_ADDRESSES;
    }

    if (command) {
        sendCommand(select, command, newAddress);
    }
}

function onChange(input) {
    const value = getInputValue(input);
    if (value !== '')
    {
        const row = $(input).parents(' tr');
        const daliType = row.find('.col-dali-type').children()[0].value;
        const address = row.find('.col-address').children()[0].value;
        const instance = (row[0].id === CONTROL_ROW_ID ? '' : daliType === DALI2 ? 'IN' : '') +
            row.find('.col-index').children()[0].value ;

        console.log(daliType, address, instance, input.name, value);

        chatSocket.send(JSON.stringify({
            type: 'set',
            message: {
                key: input.name,
                type: daliType,
                address: address,
                instance: instance,
                value: value
            }
        }));

        afterChange(input);
    }
}

function afterChange(input, ...params) {
    const row = $(input).parents(' tr');
    const daliType = row.find('.col-dali-type').children()[0].value;
    const address = row.find('.col-address').children()[0].value;
    const instance = (row[0].id === CONTROL_ROW_ID ? '' : daliType === DALI2 ? 'IN' : '') +
        row.find('.col-index').children()[0].value ;

    var forGet;
    if (address === 'B') {
        // setInputValue(input, '');
        forGet = $('#devices tbody').find('.'+daliType + ' .'+input.name).children();
    } else if (address.startsWith('G')) {
        // setInputValue(input, '');
        forGet = [];
        const group = address.substr(1);
        $('#devices tbody').find('tr.'+daliType).each(function(_, row) {
            $(row).find('[name=groups]').each(function(_, _input) {
                if (_input.value.split(',').includes(group)) {
                    forGet.push($(row).find('.' + input.name).children()[0]);
                }
            });
        });
    } else if (address !== 'BU'){
        forGet = $('#devices tbody')
            .find('.' + row.find('.col-address').children()[0].value + '.'+daliType + ' .'+input.name)
            .find('select[name='+input.name+'],input[name='+input.name+']');
    } else {
        forGet = [];
    }


    if (instance) {
        if (instance.startsWith('IN')) {
            const index = instance.substr(2);
            forGet = forGet.filter(i => {
                const td = $(forGet[i]).parents('tr').find('.index');
                return td.children().length > 0 && td.children()[0].value === index;
            });
        } else if (instance.startsWith('IT')) {
            const type = instance.substr(2);
            forGet = forGet.filter(i => {
                const td = $(forGet[i]).parents('tr').find('.type');
                return td.children().length > 0 && td.children()[0].value === type;
            });
        }
    }

    var _input;
    for (_input of forGet) setInputValue(_input, '');
    for (_input of forGet) get(_input, ...params);
}

function get(input, ...params) {
    const req = (data => chatSocket.send(JSON.stringify(data)));

    const data = {
        type: 'get',
        message: {
            key: input.name,
            type: $(input).parents(' tr').find('.col-dali-type').children()[0].value,
            address: $(input).parents(' tr').find('.col-address').children()[0].value,
            instance: 'IN' + $(input).parents(' tr').find('.col-index').children()[0].value,
            value: params.join(' ')
        }
    };

    if (input.name === 'actual_level') {
        var timerId;
        $(input).parents('tr').find('.fade_time select').each(function(i, select) {
            const timeout = parseFloat(select.options[select.selectedIndex].textContent)*1000*1.2 + 300;
            if (timeout) timerId = setTimeout(req, timeout, data);
        });

        if (timerId) return;
    }

    req(data);
}

function run(action, needHide) {
    const getColumnName = (_, input) => input.value;

    if (needHide) {
        $('#dali-count-input, #dali2-count-input').val('').trigger('change');
        for (var dali of [DALI1, DALI2]) {
            for (var address of filters[dali]) {
                $('.'+address+'.'+dali+' .value').children().hide();
            }
        }
    }
    chatSocket.send(
        JSON.stringify({
            type: action,
            message: {
                addresses: {
                    dali: filters.dali,
                    dali2: filters.dali2,
                },
                keys: {
                    common: $('.filters .columns .filter-group-common label.selected input')
                        .map(getColumnName)
                        .get(),
                    dali: $('.filter-group-dali,.filter-group-switch,.filter-group-colour')
                        .find('label.selected input')
                        .map(getColumnName)
                        .get(),
                    dali2: $('.filter-group-dali2,.filter-group-sensors,.filter-group-buttons')
                        .find('label.selected input')
                        .map(getColumnName)
                        .get(),
                },
            }
        })
    );
}

function onFilterSelect (e) {
    const cb = e.target;
    const multiSelect = $(cb).parents('.multi-select');
    multiSelect.find(' input[type=checkbox]').each(function (i, node) {
        if (node === cb) {
            const select = $(node).parents(' td.event_filter').find(' select[name=event_filter]')[0];
            select.value = (parseInt(select.value) || 0) + (1 << i) * (cb.checked ? 1 : -1);
            $(cb).parents('label').toggleClass('selected');
            // select.dispatchEvent(new Event('change'));
            return false;
        }
    });
}

function onGroupSelect (e) {
    const cb = e.target;
    const input = $(cb).parents(' td').find(' input[name=groups]')[0];
    var groups = input.value ? input.value.split(',') : [];
    if (cb.hasAttribute('group')) {
        const group = cb.getAttribute('group');

        if (cb.checked) {
            $(cb).parents('label').addClass('selected');
            if (!groups.includes(cb.group)) {
                groups.push(group);
                groups.sort((a,b) => { return parseInt(a) > parseInt(b) ? 1 : a===b ? 0 : -1; });
            }
        } else {
            $(cb).parents('label').removeClass('selected');
            if (groups.includes(group)) {
                const index = groups.indexOf(group);
                groups.splice(index, 1);
            }
        }

        input.value = groups.join(',');
    }
}

function onSceneChange (e) {
    const input = e.target;
    const index = input.getAttribute('scene');
    sendCommand(input, 'set_scene', index, input.value !== '' ? input.value : 255);
    afterChange($(input).parents(' td.scenes').children()[0], index);
}

function showMultiSelectFilters(e) {
    e.target.blur();

    const td = $(e.target.parentElement);
    const multiSelect = td.find('.multi-select');

    if (multiSelect.length === 0) {
        var type;
        const $row = td.parents('tr');
        if ($row[0].id === CONTROL_ROW_ID) {
            type = $row.find('.index')[0].children[0].value;
            if (type.startsWith('IT')) {
                type = type.substr(2);
            } else return ;
        } else {
            type = $row.find('.type')[0].children[0].value;
        }
        const d = document.createElement('div');
        const table = document.createElement('table');
        table.className = 'selectable-table';
        d.className = 'multi-select';
        e.target.parentElement.appendChild(d);
        d.appendChild(table);
        multiSelect.push(d);
        for (var i=0; i < EVENT_FILTERS[type].length; i++) {
            const label = document.createElement('label');
            const cb = document.createElement('input');
            cb.type = 'checkbox';
            if ((e.target.value & (1<<i)) > 0) {
                cb.checked = true;
                label.className = 'selected';
            }
            label.appendChild(cb);
            label.appendChild(document.createTextNode(EVENT_FILTERS[type][i]));
            cb.addEventListener('change', onFilterSelect);
            table.insertRow().insertCell().appendChild(label);
        }
    }
    if (multiSelect.css('display') === 'none') {
        const hideHandler = function (e) {
            if (td.has(e.target).length === 0) {
                multiSelect.hide();
                removeEventListener('click', arguments.callee);
                if (td.parents('tr')[0].id === CONTROL_ROW_ID) {
                    multiSelect.remove();
                }
                const select = td.find('[name=event_filter]')[0];
                if (multiSelect[0].getAttribute('hasValue') !== select.value) {
                    select.dispatchEvent(new Event('change'));
                }
            }
        };

        $('.value.' + e.target.name).find('.multi-select').each((i, node) => {
                if ($(node).css('display') === 'none') {
                    const event = new Event('click');
                    event.data = node;
                    dispatchEvent(event);
                }
            }
        );
        multiSelect.attr('hasValue', e.target.value);
        addEventListener('click', hideHandler);
        multiSelect.show();
    } else {
        const event = new Event('click');
        event.data = e.target;
        dispatchEvent(event);
    }

    return false;
}

function showMultiSelectGroups(e) {
    e.target.blur();

    const td = $(e.target.parentElement);
    const multiSelect = td.find('.multi-select');

    if (multiSelect.length === 0) {
        const $row = td.parents('tr');
        const daliType = $row.find('.col-dali-type').children()[0].value;
        const groups = e.target.value.split(',');
        const d = document.createElement('div');
        const table = document.createElement('table');

        table.className = 'selectable-table';

        d.className = 'multi-select';
        d.appendChild(table);
        e.target.parentElement.appendChild(d);
        multiSelect.push(d);

        for (var i=0; i < (daliType === DALI2 ? 8 : 4); i++) {
            const row = table.insertRow();
            for (var j = 0; j < 4; j++) {
                const cell = row.insertCell();
                const index = i * 4 + j;
                const label = document.createElement('label');
                const cb = document.createElement('input');
                cb.type = 'checkbox';
                if (groups.includes(index.toString())) {
                    cb.checked = true;
                    label.className = 'selected';
                }
                cb.setAttribute('group', index);
                label.appendChild(cb);
                label.appendChild(document.createTextNode('G' + index));
                cb.addEventListener('change', onGroupSelect);
                cell.appendChild(label);
            }
        }

        if ($row[0].id === CONTROL_ROW_ID) {
            const row = table.insertRow();
            for (var attrs of [['Add', 'add_to_group'], ['Remove', 'remove_from_group']]) {
                const cell = row.insertCell();
                cell.colSpan = 2;
                const btn = document.createElement('button');
                btn.id = attrs[1];
                btn.onclick = (e) => {
                    const event = new Event('click');
                    event.data = e.target;
                    dispatchEvent(event);
                };
                btn.style.width = '100%';
                btn.appendChild(document.createTextNode(attrs[0]));
                cell.appendChild(btn);
            }
        }
    }
    if (multiSelect.css('display') === 'none') {
        const hideHandler = function (e) {
            if (td.has(e.target).length === 0) {
                multiSelect.hide();
                removeEventListener('click', arguments.callee);
                const input = td.find('[name=groups]')[0];
                const was = multiSelect[0].getAttribute('hasValue').split(',');
                const will = input.value ? input.value.split(',') : [];

                if (td.parents('tr')[0].id === CONTROL_ROW_ID) {
                    input.value = '';
                    if (e.data.id) {
                        sendCommand(input, e.data.id, ...will);
                        afterChange(input);
                    }
                } else {
                    if (multiSelect[0].getAttribute('hasValue') !== input.value) {
                        const toAdd = will.filter(i => i !== '' && !was.includes(i));
                        const toRemove = was.filter(i => i !== '' && !will.includes(i));

                        if (toAdd.length > 0) sendCommand(input, 'add_to_group', ...toAdd);
                        if (toRemove.length > 0) sendCommand(input, 'remove_from_group', ...toRemove);

                        if (toAdd.length > 0 || toRemove.length > 0) {
                            afterChange(input);
                        }
                    }
                }
                multiSelect.remove();
            }
        };
        $('.value.' + e.target.name).find('.multi-select').each((i, node) => {
                if ($(node).css('display') === 'none') {
                    const event = new Event('click');
                    event.data = node;
                    dispatchEvent(event);
                }
            }
        );
        multiSelect.attr('hasValue', e.target.value);
        addEventListener('click', hideHandler);
        multiSelect.show();
    } else {
        const event = new Event('click');
        event.data = e.target;
        dispatchEvent(event);
    }

    return false;
}

function showMultiSelectScenes(e) {
    e.target.blur();

    const td = $(e.target.parentElement);
    const multiSelect = td.find('.multi-select');

    if (multiSelect.length === 0) {
        const scenes = JSON.parse(e.target.value);
        const d = document.createElement('div');
        const table = document.createElement('table');

        table.className = 'selectable-table';

        d.className = 'multi-select';
        d.appendChild(table);
        e.target.parentElement.appendChild(d);
        multiSelect.push(d);

        for (var i=0; i < 4; i++) {
            const row = table.insertRow();
            for (var j = 0; j < 4; j++) {
                const cell = row.insertCell();
                const index = i * 4 + j;
                const label = document.createElement('label');
                label.appendChild(document.createTextNode(index + ':'));
                const input = document.createElement('input');
                input.type = 'number';
                input.placeholder = 'MASK';
                input.min = '0';
                input.max = '254';
                input.value = scenes[index] === 255 ? '' : scenes[index];
                input.setAttribute('scene', index);
                label.appendChild(input);
                input.addEventListener('change', onSceneChange);
                cell.appendChild(label);
            }
        }
    }
    if (multiSelect.css('display') === 'none') {
        const hideHandler = function (e) {
            if (td.has(e.target).length === 0) {
                multiSelect.hide();
                removeEventListener('click', arguments.callee);
                multiSelect.remove();
            }
        };
        $('.value.' + e.target.name).find('.multi-select').each((i, node) => {
                if ($(node).css('display') === 'none') {
                    const event = new Event('click');
                    event.data = node;
                    dispatchEvent(event);
                }
            }
        );
        addEventListener('click', hideHandler);
        multiSelect.show();
    } else {
        const event = new Event('click');
        event.data = e.target;
        dispatchEvent(event);
    }

    return false;
}

function onControlDaliSelect(e) {
    const row = $(e.target).parents('tr');

    switch (e.target.value) {
        case DALI1:
            $(row).find('.col-index select').hide();
            $(row).find('.col-address select').each((_, select) => {
                select.value = '';
                select.dispatchEvent(new Event('change'));
                $(select).show();
                $(select).find(' optgroup#groups-dali2').hide();
                // $(select).find(' optgroup#groups-addresses').hide();
            });
            break;

        case DALI2:
            $(row).find('.col-index select').show();
            $(row).find('.col-address select').each((_, select) => {
                select.value = '';
                select.dispatchEvent(new Event('change'));
                $(select).show();
                $(select).find(' optgroup#groups-dali2').show();
                // $(select).find(' optgroup#groups-addresses').show();
            });
            break;
        default:
            $(row).find('.col-index select,.col-address select').each((_, select) => {
                select.value = '';
                select.dispatchEvent(new Event('change'));
                $(select).hide();
            });
            break;
    }
}

function onControlIndexSelect(e) {
    if (e.target.value === '') {
        $(e.target).parents('tr').find('.col-dali2 select, .col-sensors select, .col-buttons select').attr('disabled', 'disabled');
    } else {
        $(e.target).parents('tr').find('.col-dali2 select, .col-sensors select, .col-buttons select').removeAttr('disabled');
    }
}

function onControlAddressSelect(e) {
    const row = $(e.target).parents('tr');
    const daliType = row.find('.col-dali-type').children()[0].value;

    if (e.target.value === '') {
        $(row).find('.col-dali select, .col-dali input, .col-dali2 select, .col-sensors select, .col-buttons select, .col-common select, .col-common input').attr('disabled', 'disabled');
        $(row).find('.col-index select').each((_, select) => {
            select.value = '';
            select.dispatchEvent(new Event('change'));
            $(select).hide();
        });
    } else if (daliType === DALI1) {
        $(row).find('.col-dali select, .col-dali input, .col-common select, .col-common input').removeAttr('disabled');
        $(row).find('.col-dali2 select, .col-sensors select, .col-buttons select').attr('disabled', 'disabled');
    } else if (daliType === DALI2) {
        $(row).find('.col-common select, .col-common input').removeAttr('disabled');
        $(row).find('.col-dali select .col-dali input').attr('disabled', 'disabled');
        $(row).find('.col-index select').show();
    }
}

function updateStripped() {
    $('#devices-table tbody tr:visible').each((i, row) => {
        if (i%2 === 0) {
            $(row).addClass('dark');
        } else {
            $(row).removeClass('dark');
        }

        $(row).find('.col-number').text(i+1);
    });
}