const ADD = 'add';
const REMOVE = 'remove';

const NAME = 'name';
const INSTANCE = 'instance';
const ADDRESS = 'address';
const MARK = 'mark';
const TOPIC = 'topic';

var chatSocket;

var counter = 0;
var prefixes = [];
var filters = {address: [], instance: [], mark: [], name: [], topic: []};
var columns = [];


$(document)
    .on('click', '.panel-heading', function () {
        var $this = $(this);
        $this.parents('.filters .panel').find('.panel-body').slideToggle();
        $this.find('i').toggleClass('glyphicon-chevron-down').toggleClass('' +
            'glyphicon-chevron-up');
    })
    .on('click', '.filter-btn', function () {
        $(document).find('.filters').slideToggle();
    });

$(document).ready(function () {
    setPlay();
});

function setAddress(e) { setFilter(e, ADDRESS); }
function setInstance(e) { setFilter(e, INSTANCE); }
function setMark(e) { setFilter(e, MARK); }
function setName(e) { setFilter(e, NAME); }

function setFilter(cb, filter) {
    if (cb.value) {
        if (cb.checked) {
            filters[filter].push(cb.value);
            cb.labels[0].classList.add('selected');
        } else {
            const index = filters[filter].indexOf(cb.value);
            if (index > -1) filters[filter].splice(index, 1);
            cb.labels[0].classList.remove('selected');
        }
        $(cb).parents('.panel').find('.panel-heading span.badge').text(filters[filter].length);

        dispatchEvent(new CustomEvent('filter', {detail: {type: filter, items: filters[filter]}}));

        if (filter === TOPIC && cb.value) {
            sendAction(cb.checked ? ADD : REMOVE, cb.value);
        }
    }
}

function onFilterInput(input) {
    const valid = input.checkValidity();
    $(input.parentElement).find(' input[type=checkbox]').each(function () {
        if (this.checked) this.click();
        this.value = valid ? input.value.trim() : null;
    });
}

function sendAction(action, part) {
    var topics = [];
    if (part) {
        for (var i=0; i < prefixes.length; i++)
            topics.push(prefixes[i] + part);

        chatSocket.send(JSON.stringify({message: {type: action, topics: topics}}));
    }
}

function setColumn(cb) {
    if (columns.indexOf(cb.value) !== -1) columns.splice(columns.indexOf(cb.value), 1);
    else columns.push(cb.value);

    $(cb).parents('.panel').find('.panel-heading span.badge').text(columns.length);
    cb.labels[0].classList.toggle('selected');

    document.querySelectorAll(`.${cb.value}`).forEach(cell => {
        cell.classList.toggle('hidden');
    });
}

function comparePart(part) {
    return true;
    if (filters[TOPIC].indexOf(part) !== -1) return true;
    else {
        for (var i=0; i < filters[TOPIC].length; i++) {
            if (filters[TOPIC][i].indexOf('#') !== -1) {
                const index = filters[TOPIC][i].indexOf('#');
                return part.substr(0, index) === part.substr(0, index)
            } else if (filters[TOPIC][i].indexOf('+') !== 0) {
                const _parts = part.split('/');
                const items = filters[TOPIC][i].split('/');
                while (items.indexOf('+') !== -1) {
                    _parts.splice(items.indexOf('+'), 1);
                    items.splice(items.indexOf('+'), 1);
                }
                if (_parts.join(' ') === items.join(' ')) return true;
            }
        }
    }

    return false;
}

function setPause() {
    $('span.glyphicon-pause').hide();
    $('span.glyphicon-play').show();
    chatSocket.onclose = null;
    chatSocket.close();
}

function setPlay() {
    chatSocket = new WebSocket('ws://' + window.location.host + SOCKET_URL);

    chatSocket.onclose = function(e) { alert('Соединение с сервером разорвано'); };
    chatSocket.onopen = function() {
        console.log('Opened as ' + SOCKET_URL);
        $('span.glyphicon-pause').show();
        $('span.glyphicon-play').hide();
    };
    chatSocket.onmessage = function (e) {
        const data = JSON.parse(e.data);
        if (data.hasOwnProperty('message')) {
            const message = data.message;
            if (comparePart(message[TOPIC])) {
                const owner = $('.content table')[0];
                if (owner) {
                    const row = owner.insertRow(1);
                    if (counter++ % 2 === 0) row.classList.add('dark');
                    if (message.direct === 'Send') row.classList.add('send-item');
                    addTd('col-time', message.time);
                    addTd('col-hex', message.hex);
                    addTd('col-addr', message.address);
                    addTd('col-ins', message.instance);
                    addTd('col-desc', message.desc);
                    row['mark'] = message.mark;
                    row['name'] = message.name;
                    row['address'] = message.address;
                    row['instance'] = message.instance;
                    row['desc'] = message.desc;
                    row['filtered'] = [];

                    function checkFilter(name) {
                        if (message[name] && filters[name].length > 0 && filters[name].indexOf(message[name]) === -1) {
                            row['filtered'].push(name);
                        }
                    }

                    function addTd(className, value) {
                        const td = row.insertCell();
                        td.textContent = value;
                        $(td).addClass(className);
                        if (columns.indexOf(className) !== -1) td.classList.add('hidden');
                    }

                    checkFilter(MARK);
                    checkFilter(ADDRESS);
                    checkFilter(INSTANCE);
                    checkFilter(NAME);

                    row.style.display = row['filtered'].length > 0 ? 'none' : '';

                    addEventListener('filter', function (e) {
                        if (row.hasOwnProperty(e.detail.type)) {
                            const fIndex = row.filtered.indexOf(e.detail.type);
                            if (e.detail.items.length === 0 || e.detail.items.indexOf(row[e.detail.type]) !== -1) {
                                if (fIndex !== -1) row.filtered.splice(fIndex, 1);
                                if (row.filtered.length === 0) row.style.display = '';
                            } else {
                                row.style.display = 'none';
                                if (fIndex === -1) row.filtered.push(e.detail.type);
                            }
                        }
                    });

                    if (owner.rows.length > 1000) {
                        owner.deleteRow(-1);
                    }
                }
            }
        }
    };
}