"""reface URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
import json
import os

from django.urls import re_path, include
from django.views.i18n import JavaScriptCatalog
from django.contrib import admin

from .dashboard import views, monitor

try:
    with open(os.environ.get('REFACE_CONFIG'), 'r') as fs:
        HTTP_ROOT = json.load(fs)['http_root']
except:
    HTTP_ROOT = 'rapida'

B = r'(?P<box_id>\d+)'
M = r'(?P<module_id>\d+)'
C = r'(?P<channel_id>\d+)'
DALI_TYPE = r'(?P<dali_type>\w+)'


urlpatterns = [
    re_path(f'{HTTP_ROOT}/box/{B}/', views.box,                                         name='box'),
    re_path(f'{HTTP_ROOT}/{B}/{M}/{C}/builder/process/', views.line_builder_process,    name='building_process'),
    re_path(f'{HTTP_ROOT}/{B}/{M}/{C}/builder/', views.line_builder),
    re_path(f'{HTTP_ROOT}/{B}/{M}/{C}/ordering/{DALI_TYPE}/', views.ordering,           name='ordering'),
    re_path(f'{HTTP_ROOT}/{B}/{M}/{C}/probation/', views.probation),
    re_path(f'{HTTP_ROOT}/{B}/{M}/{C}/devices/', views.devices,                         name='devices'),
    re_path(f'{HTTP_ROOT}/{B}/{M}/{C}/monitor/', monitor.Dali.as_view()),
    re_path(f'{HTTP_ROOT}/{B}/{M}/{C}/', views.channel,                                 name='channel'),
    re_path(f'{HTTP_ROOT}/{B}/{M}/', views.module,                                      name='module'),
    re_path(f'{HTTP_ROOT}/{B}/set_name', views.set_box_name,                            name='set_box_name'),
    re_path(f'{HTTP_ROOT}/configure/', views.configure,                                 name='configure'),
    re_path(f'{HTTP_ROOT}/0', views.controller,                                         name='controller'),
    re_path(f'{HTTP_ROOT}/reload/', views.reload,                                       name='reload'),
    re_path(f'{HTTP_ROOT}/add_box/', views.add_box,                                     name='add_box'),
    re_path(f'{HTTP_ROOT}/set_name/', views.set_name,                                   name='set_controller_name'),
    re_path(f'{HTTP_ROOT}/upgrade/', views.upgrade,                                     name='upgrade'),
    re_path(f'{HTTP_ROOT}/jsi18n/', JavaScriptCatalog.as_view(),                        name='javascript-catalog'),
    re_path(f'{HTTP_ROOT}/admin/', admin.site.urls),
    re_path(f'{HTTP_ROOT}/auth/', include('django.contrib.auth.urls')),
    re_path(f'{HTTP_ROOT}/', views.root,                                                name='root'),
]
