const current = $('#current')[0];
const wanted = $('#wanted')[0];
const COM = 0;
const CPL = 1;
const CUR = 2;
const levels = [
    $('#common-level'),
    $('#complete-level'),
    $('#current-level')
];

const SND_SUCCESS = new Audio($('#sound-success').attr('src'));
const SND_NAVI = new Audio($('#sound-navi').attr('src'));

var chatSocket, repeatHandlerId;

function setCommonLevels(input) {
    onLevelInput(input);
    setCookie(input.id, input.value);

    if (started) {
        if (levels[COM][0] === input) {
            $(wanted).find('option').each((_, opt) => setLevel(opt.value, COM));
        } else if (levels[CPL][0] === input) {
            completes.forEach(addr => setLevel(addr, CPL));
        } else if (levels[CUR][0] === input) {
            if (current.value) {
                setLevel(current.value, CUR);
            }
        }
    }
}

function setRepeatInterval(direction) {
    clearInterval(repeatHandlerId);
    repeatHandlerId = undefined;
    if (direction !== undefined) {
        repeatHandlerId = setInterval(setCurrent, $('#repeat-delay').val()*1000, direction);
    }
}

let pressedKey;
addEventListener('mouseup', mouseUp);

addEventListener('keyup', (e) => {
    if (e.code === pressedKey) {
        pressedKey = undefined;
        setRepeatInterval();
    }
});

addEventListener('keydown', (e) => {
    if (pressedKey)
        return;
    if (e.code === 'ArrowUp' || e.code === 'ArrowRight') {
        setCurrent(1);
        setRepeatInterval(1);
    } else if (e.code === 'ArrowLeft' || e.code === 'ArrowDown') {
        setCurrent(-1);
        setRepeatInterval(-1);
    } else if (e.code === 'Enter') {
        setAddress();
    } else if (e.code === 'Escape') {
        setRepeatInterval();
    } else return;

    pressedKey = e.code;

    e.preventDefault();
});

let pressedBtn;
function mouseUp(e) {
    clearTimeout(longTimeout);
    if (pressedBtn) {
        setRepeatInterval();
        setTimeout(() => {
                $(pressedBtn).css('pointer-events', '');
                pressedBtn = undefined;
            }, 300
        );
    }
}

let longTimeout;
function mouseDown(target, i) {
    longTimeout = setTimeout(() => {
        $(target).css('pointer-events', 'none');
        pressedBtn = target;
        setRepeatInterval(i);
    }, 500);
}

function setCurrent(i, verbose) {
    const last = current.oldValue || current.value;
    current.oldValue = undefined;
    current.selectedIndex =
        current.selectedIndex+i >= current.options.length ? 0 :
            current.selectedIndex+i < 0 ? current.options.length - 1 :
                current.selectedIndex + i;

    if (last !== '') setLevel(last, COM);
    setLevel(current.value, CUR);
    setHistory('current', current.value);
    if(!verbose) playSound(SND_NAVI);
    started = true;
}

function setLevel(address, level) {
    if (DALI_TYPE === DALI2 && level !== CUR) return;

    send({
        type: 'set_current',
        message: {
            command: 'set_current',
            type: DALI_TYPE,
            address: 'D' + address,
            params: [levels[level].val()]
        }
    });
}

function setAddress() {
    setLevel(current.value, CPL);
    if (current.value !== wanted.value) {
        setLevel(wanted.value, COM);
        send({
            type: 'set_address',
            message: {
                type: DALI_TYPE,
                address: 'D' + current.value,
                params: ['D' + wanted.value]
            }
        });
    }

    $(current).find('option[value='+wanted.value+']').remove();
    current.selectedIndex = 0;
    completes.push(wanted.value);

    const hasWantedIndex = wanted.selectedIndex;
    wanted.options.remove(hasWantedIndex);
    wanted.selectedIndex = hasWantedIndex < wanted.options.length ? hasWantedIndex : 0;

    if (wanted.options.length === 0) {
        console.log('Complete');
        setHistory('current');
        setHistory('wanted');
        setHistory('completes');
        chatSocket.close();
    } else {
        setHistory('wanted', wanted.value);
        setHistory('completes', completes.join(','));
        setCurrent(0, true);
    }

    playSound(SND_SUCCESS);
}

$(document).ready(function () {
    chatSocket = new WebSocket('ws://' + window.location.host + '/rapida/ws/ordering/' + room + '/');

    chatSocket.onclose = function(e) { alert('Соединение с сервером разорвано'); };
    chatSocket.onopen = function() {
        console.log('Opened');
    };
    chatSocket.onmessage = function (e) {
        console.log(e);
    };

    $(document).on('click', '.panel-heading', function () {
        var $this = $(this);
        $this.parents('.panel').find('.panel-body').slideToggle();
        const shown = $this.find('i')
            .toggleClass('glyphicon-chevron-down')
            .toggleClass('glyphicon-chevron-up')
            .hasClass('glyphicon-chevron-up');

        if (shown) {
            eraseCookie('settings-hidden');
        } else {
            setCookie('settings-hidden', 1);
        }
    });
});

const send = (data) => {
    chatSocket.send(
        JSON.stringify(data)
    );
};

const onLevelInput = (input) => input.labels[0].textContent = input.value;

const  start = () => {
    $('#work-table').show();
    $('.start-action').toggle();
    current.selectedIndex = wanted.selectedIndex = 0;

    if (DALI_TYPE === DALI1) {
        for (var option of wanted.options) {
            setLevel(option.value, COM);
        }
    }

    setCurrent(0);
};

const  restart = () => {
    $(wanted).find('option').each((_, opt) => setLevel(opt.value, COM));
    completes.forEach(addr => setLevel(addr, COM));
    setLevel(0, CUR);
    setHistory('completes');
    setHistory('current', 0);
    setHistory('wanted');
    location.reload();
};

function setHistory(name, value) {
    const url = new URL(location.href);
    if (value === undefined || value === '') {
        url.searchParams.delete(name);
    } else {
        url.searchParams.set(name, value);
    }
    history.replaceState('', '', '?' + url.searchParams.toString())
}

function playSound(sound) {
    sound.pause();
    sound.currentTime = 0;
    sound.play();
}