import json
import os
import re

from resh.domains._domain import Domain
from resh.domains.gateway import GatewayUtil
from ..templates.gateways import rgp_server_sv_config
from ..utils import tools
from ..utils.const import BRIDGE, PORT
from ..utils.errors import BadParameter, StopExec


class BridgeDomain(Domain, GatewayUtil):
    def_params = {}
    domain_name = BRIDGE

    LOCAL_PORT = 'localPort'
    LOCAL_HOST = 'localHost'
    LOCAL_CAN_INTERFACE = 'canInterface'

    RGP_BRIDGE_MASK = 'server'
    BRIDGE_SV_MASK = 'rgp_server'

    DEFAULT_BRIDGE_CONFIG = {
        "logLevel": 1,
        "canInterface": "can0",
        "localHost": "0.0.0.0"
    }

    def exec(self):
        raise NotImplemented()

    @classmethod
    def add_bridge_config(cls, **kwargs):
        """ Добавить конфигурацию нового bridge """
        template = {
            cls.LOCAL_PORT: cls.DEFAULT_PORT
        }
        template.update(cls.DEFAULT_BRIDGE_CONFIG)
        template.update(kwargs)

        file_name = os.path.join(cls.SPREAD_SERVICES_CONF_DIR, f'{cls.RGP_BRIDGE_MASK}{cls.SPREAD_SERVICES_EXT}')
        if os.path.exists(file_name):
            raise StopExec(f"Bridge already exist!")
        tools.write_file(file_name, json.dumps(template, indent=4))

    @classmethod
    def remove_bridge_config(cls):
        """ Удалить конфигурацию bridge """
        file_name = f'{cls.SPREAD_SERVICES_CONF_DIR}{cls.RGP_BRIDGE_MASK}{cls.SPREAD_SERVICES_EXT}'
        if os.path.exists(file_name):
            os.remove(file_name)


class status(BridgeDomain):
    def exec(self):
        res = dict(status=False)
        sv_conf = os.path.join(self.SUPERVISOR_CONF_DIR, f'{self.BRIDGE_SV_MASK}{self.SUPERVISOR_CONF_EXT}')
        if os.path.exists(sv_conf):
            rgp_conf = os.popen(f'cat {sv_conf} | grep ^command=').read().strip()
            rgp_conf = re.split(r'command=.+\s+--config\s+', rgp_conf)[1].split(' ')[0].strip()
            if os.path.isfile(rgp_conf):
                with open(rgp_conf, 'r') as file:
                    rgp_conf = json.loads(file.read())

            res.update({
                self.LOCAL_HOST: rgp_conf.get(self.LOCAL_HOST, 'unknown'),
                self.LOCAL_PORT: rgp_conf.get(self.LOCAL_PORT, 'unknown'),
                self.LOCAL_CAN_INTERFACE: rgp_conf.get(self.LOCAL_CAN_INTERFACE, 'unknown'),
                'status': 'RUNNING' in os.popen('sudo supervisorctl status rgp_server').read().strip().split()
            })

            return res


class start(BridgeDomain):
    var_params = dict(list({PORT: GatewayUtil.DEFAULT_PORT}.items()) + list(BridgeDomain.DEFAULT_BRIDGE_CONFIG.items()))

    def exec(self):
        bridge_kwargs = {}

        if self.has_next and isinstance(self.next_param_recursive, int):
            bridge_kwargs[self.LOCAL_PORT] = self.next_param

        while self.has_next:
            pair = self.next_param
            try:
                key, value = pair.split('=')
                if key in self.DEFAULT_BRIDGE_CONFIG:
                    bridge_kwargs[key] = value
            except BaseException:
                raise BadParameter(pair)

        self.add_bridge_config(**bridge_kwargs)
        self.add_sv_config(f'{self.BRIDGE_SV_MASK}{self.SUPERVISOR_CONF_EXT}', rgp_server_sv_config.template)

    @classmethod
    def example(cls):
        return ' '.join([cls.domain_name, cls.__name__] +
                        [f'{cls.def_params[p]}' for p in cls.def_params] +
                        [f'{cls.opt_params[p]}' for p in cls.opt_params] +
                        [f'[{"{}=".format(p) if p != PORT else ""}{cls.var_params[p]}]' for p in cls.var_params])


class stop(BridgeDomain):
    def exec(self):
        self.remove_sv_config(f'{self.BRIDGE_SV_MASK}{self.SUPERVISOR_CONF_EXT}')
        self.remove_bridge_config()


class up(BridgeDomain):
    def exec(self): tools.sys(f'sudo supervisorctl start {self.BRIDGE_SV_MASK}')


class down(BridgeDomain):
    def exec(self): tools.sys(f'sudo supervisorctl stop {self.BRIDGE_SV_MASK}')
