# import mercury

from resh.domains._domain import Domain
from resh.protocols.can_utils.modbusi_packet import ModbusSend
from resh.utils import tools
from resh.utils.errors import DeviceAddressError, ChannelAddressError
from ..utils.const import *


class Mercury(Domain):
    def_params = {MODULE: 1, CHANNEL: 2, DEVICE_ADDR: '30'}
    domain_name = MERCURY

    def __init__(self, *args):
        super(Mercury, self).__init__(*args)
        self._channel = None
        self._address = None
        self._address_raw = None

    def exec(self):
        raise NotImplemented()

    @property
    def channel(self):
        if self._channel is None:
            if len(self._args) < 2:
                raise ChannelAddressError()
            else:
                self._channel = tools.parse_channel(self._args[1])
        return self._channel

    @property
    def address(self):
        if not self._address:
            try:
                self._address = int(self._args[2])
            except BaseException:
                raise DeviceAddressError()
        return self._address

    def _send(self, end_frame, need_answer=False):
        """Выполнение команды и необходимых подготовок с самостоятельным изъятием параметров их стека"""

        msg = ModbusSend(self.module, self.channel, end_frame)
        return msg.send(need_answer=need_answer)

#     def send(self, cmd):
#         """Простое выполнение команды переданной как экземпляр класса"""
#
#         log.info(f'-> {str(cmd)}')
#         if cmd.access_level > 0:
#             access_cmd = mercury.commands.CheckConnect(cmd.addr_int)
#             self._send(access_cmd.pack[:-2], need_answer=cmd.has_response)
#         result = self._send(cmd.pack[:-2], need_answer=cmd.has_response)
#         if cmd.has_response:
#             cmd.set_response(result)
#             return cmd.response
#
#
# """https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Modbus#Base"""
#
#
# class query_serial(Mercury):
#     def exec(self): return self.send(mercury.commands.SerialCommand(self.address)).value
#
#
# class query_version(Mercury):
#     def exec(self): return self.send(mercury.commands.Version(self.address)).value
