from ..utils.const import *


class Loc:
    INTERACTIVE_HELLO = 'Интерактивный режим' + '\n' + \
                        '  Для выхода введите команду "exit" или нажмите Ctrl-D' + '\n' + \
                        '  Для получения справки введите команду "help"'

    """Ошибки"""
    NONEXISTENT_LOC = 'Отсутствующая локализация для '
    BAD_MODULE_ADDRESS = 'Адрес модуля не указан или указан неверно!'
    BAD_GATEWAY_INDEX = 'Номер шлюза не указан или указан неверно!'
    BAD_CHANNEL = 'Номер канала не указан или указан неверно!'
    BAD_DEVICE_ADDRESS = 'Адрес устройства не указан или указан неверно!'
    BAD_INSTANCE = 'Instance не указан или указан неверно!'
    UNAVAILABLE_DOMAIN = 'Недоступный домен "{}"'
    UNAVAILABLE_METHOD = 'Недоступная команда "{}"'
    PARAMETER_ERROR = 'Ошибка ввода параметра(ов)'
    UNAVAILABLE_INSTANCE = 'Неприменимо для указанного типа'

    """Параметры"""
    ADDRESS_IS_BUSY = 'Адрес({}) уже используется! Используйте "swap_short_addresses" для замены адресов'

    """Интерфейс"""
    IDENTIFY_QUESTION = 'Для завершения режима идентификации нажмите клавишу "Enter"'
    CMD_USAGE = 'Использование: '
    CMD_EXAMPLE = 'Пример: '

    """Общие параметры"""
    G_LEVEL = 'Уровень яркости (целое 0..254 или строка MASK)'
    G_BYTE = '0..255 или 0x0..FF'
    G_SEND_TWICE = 'необязательный параметр, его наличие указывает необходимость двойной отправки'
    G_MODE = f'{MODE_NOCROSS}/{MODE_CROSS} - запретить/разрешить пересечение адресов dali и dali2'
    G_START_ADDRESS = 'начинать адресацию устройств с указанного адреса. По умолчанию - 0'
    G_DTR0_VALUE = 'значение, записываемое в DTR0'
    G_DUMP = 'Запрашивает все параметры одного устройства'
    G_IP_TYPE = 'строка формата "ddd.ddd.ddd.ddd", где ddd - целое [0..255]'
    G_ADDRESS = 'D0..63 - адрес устройства'
    G_GROUP_ID_0_31 = '0..31 - идентификатор группы'

    """Общие команды"""
    G_EXTEND_LINE = 'Осуществляет расширение линии'
    G_CONSTRUCT_LINE = 'Осуществляет сбор линии'
    G_SCAN_LINE = 'Ищет на линии устройства с короткими адресами и запрашивает их тип'
    G_FIX_DOUBLE_ADDRESSES = 'Поиск и разрешение задвоенных адресов'

    _loc = None

    def loc(self):
        if self._loc is None:
            from resh.domains import system, module, dali1, dali2, gateway, bridge, config

            self._loc = {
                DEF_PARAMS: {
                    CLASS: 'класс модуля (0 - любой, 1 - Dali, 2 - Modbus)',
                    MODULE_0_31: '1..29 - адрес модуля; 0 - все модули; 30 - блок питания; 31 - контроллер',
                    MODULE: 'адрес модуля (целое 1..29)',
                    CHANNEL: 'номер канала (целое 1..8)',
                    DEVICE_ADDR: 'D0..63 - адрес устройства; G0..15 - адрес группы; B - все устройства; BU - все безадресные;',
                    INSTANCE: 'T{x} - устройства типа {x}; N{x} - устройства с индексом {x}; B - все типы;',
                    GATEWAY_VAR: 'id шлюза (целое 1..)'},
                MODULE_DOMAIN: {
                    module.raw_command.__name__: {
                        DESC: 'Выполнение "сырой" команды, составленной из передаваемых параметров',
                        BYTES_ARR: 'байты, составляющие тело команды, через пробел'},
                    module.raw_query.__name__: {
                        DESC: 'Выполнение "сырого" запроса, составленной из передаваемых параметров',
                        BYTES_ARR: 'байты, составляющие тело запроса, через пробел'},
                    module.version.__name__: {
                        DESC: 'Версия прошивки модуля'},
                    module.serial.__name__: {
                        DESC: 'Серийный номер модуля'},
                    module.set_address.__name__: {
                        DESC: 'Установка модулю нового адреса',
                        NEW_ADDRESS: '<int> новый адрес модуля в диапозоне 1..29'},
                    module.update_firmware.__name__: {
                        DESC: 'Обновление прошивки модуля',
                        FILE: 'абсолютный путь до файла прошивки'},
                    module.update_power_firmware.__name__: {
                        DESC: 'Обновление прошивки модуля питания',
                        FILE: 'абсолютный путь до файла прошивки'},
                    module.reboot.__name__: {
                        DESC: 'Перезагрузка модуля'},
                    module.find.__name__: {
                        DESC: 'Ищет все интерфейсные модули, подключенные к шине.'},
                    module.readdress.__name__: {
                        DESC: 'Переадресует в автоматическом режиме все интерфейсные модули, подключенные к шине.',
                        FORCED: 'Если ключ указан, то команды отправляются без обработки'},
                    module.info.__name__: {
                        DESC: 'Запрашивает информацию об интерфейсном модуле'},
                    module.dali_status.__name__: {
                        DESC: 'Запрашивает статус интерфейсного модуля и его каналов'},
                    module.monitor.__name__: {
                        DESC: 'Монитор пакетов в CAN-шине с возможностью ограничения по фильтрам',
                        MODULE: 'адрес модуля (целое 1..29)',
                        CHANNEL: 'номер канала (целое 1..8)',
                        DEVICE_ADDR: 'D0..63 - адрес устройства; G0..15 - адрес группы;',
                        INSTANCE: 'T{x} - устройства типа {x}; N{x} - устройства с индексом {x};',
                        NAME: 'имя команды и/или группы команд (SYSTEM, EVENT, DALI, DALI2)',
                        NAMED_PARAMS_DESC: 'Несколько значений для каждого параметра могут быть указаны через ","',
                        EXCLUDE_PARAMS_DESC: 'Значение параметра со снаком "-" используется для исключения из вывода',
                        BROADCAST_VALUE: '"0" в качестве параметра означает любое выражение'
                    },
                    module.modbus_channel_status.__name__: {
                        DESC: 'Запрашивает настройки канала modbus'},
                    module.modbus_set_channel_conf.__name__: {
                        DESC: 'Устанавливает настройки канала modbus',
                        BAUD: 'скорость приема-передачи (1200/2400/4800/9600/19200/38400/57600/115200/230400/460800/921600)',
                        PARITY: 'контроль четности (none/even/odd)',
                        STOP_BITS: 'число стоп-битов (1 или 2)'},
                    module.modbus_reset_channel.__name__: {
                        DESC: 'Сброс настройки канала'}
                },
                CONFIG: {
                    config.reset.__name__: {
                        DESC: 'Конфигурирование окружения с ПОТЕРЕЙ ВСЕХ ДАННЫХ '},
                    config.update.__name__: {
                        DESC: 'Конфигурирование окружения с сохранением данных'},
                    config.update_nginx.__name__: {
                        DESC: 'Обновление файла конфигурации для nginx'},
                    config.update_bash.__name__: {
                        DESC: 'Обновление файла автозаполнения для командной строки'},
                },
                SYSTEM: {
                    system.get_device_type.__name__: {
                        DESC: 'Читает тип исполнения контроллера'},
                    system.get_mac.__name__: {
                        DESC: 'Запрашивает MAC-адрес сетевого контроллера'},
                    system.get_ip.__name__: {
                        DESC: 'Запрашивает конфигурацию сетевого интерфейса'},
                    system.set_ip.__name__: {
                        DESC: 'Изменяет IP-адрес на заданный (если указаны маска подсети и шлюз, то изменяет и их)',
                        IP: self.G_IP_TYPE,
                        MASK_LC: self.G_IP_TYPE,
                        GATEWAY_VAR: self.G_IP_TYPE},
                    system.get_dns.__name__: {
                        DESC: 'Запрашивает DNS-конфигурацию сетевого интерфейса'},
                    system.set_dns.__name__: {
                        DESC: 'Изменяет адрес DNS-сервера на заданный (через пробел можно указать адрес второго DNS-сервера)',
                        DNS: self.G_IP_TYPE,
                        DNS2: self.G_IP_TYPE},
                    system.get_date_time.__name__: {
                        DESC: 'Запрашивает текущие дату, время и зону'},
                    system.set_date.__name__: {
                        DESC: 'Изменяет текущую дату на заданную (если год не указан, то подразумевается текущий год)',
                        DAY: 'целое [1..31]',
                        MONTH: 'целое [1..12]',
                        YEAR: 'целое [1970..2050]'},
                    system.set_time.__name__: {
                        DESC: 'Изменяет текущее время на заданное (если секунды не указаны, то подразумевается ноль секунд)',
                        HOURS: 'целое [0..23]',
                        MINUTES: 'целое [0..59]',
                        SECONDS: 'целое [0..59]'},
                    system.set_time_zone.__name__: {
                        DESC: 'Изменяет текущую зону на заданную',
                        TZ: 'целое [-14..12]'},
                    system.get_info.__name__: {
                        DESC: 'Запрашивает информацию о характеристиках системы'},
                    system.get_status.__name__: {
                        DESC: 'Запрашивает информацию о статусе устройств системы'},
                    system.reboot.__name__: {
                        DESC: 'Перегружает контроллер'},
                },
                DALI: {
                    dali1.raw_command.__name__: {
                        DESC: 'Формирует и отправляет двухбайтный DALI-пакет',
                        BYTE_ADDR: 'адрес DALI-устройства (' + self.G_BYTE + ')',
                        BYTE_OPCODE: 'значение для формирования второго байта "Opcode byte" DALI-пакета (' + self.G_BYTE + ')',
                        SEND_TWICE: self.G_SEND_TWICE},
                    dali1.raw_command_dtr0.__name__: {
                        DESC: 'Формирует и отправляет двухбайтный DALI-пакет, загружая в регистр DTR0 значение <Dtr0Value>',
                        DTR0_VALUE: 'значение, записываемое в регистр DTR0 (' + self.G_BYTE + ')',
                        BYTE_ADDR: 'адрес DALI-устройства (' + self.G_BYTE + ')',
                        BYTE_OPCODE: 'значение для формирования второго байта "Opcode byte" DALI-пакета (' + self.G_BYTE + ')',
                        SEND_TWICE: self.G_SEND_TWICE},
                    dali1.raw_query.__name__: {
                        DESC: 'Формирует и отправляет двухбайтный DALI-пакет, ожидается однобайтный ответ',
                        BYTE_ADDR: 'адрес DALI-устройства (' + self.G_BYTE + ')',
                        BYTE_OPCODE: 'значение для формирования второго байта "Opcode byte" DALI-пакета (' + self.G_BYTE + ')'},
                    dali1.raw_read_memory.__name__: {
                        DESC: 'Чтение значения из указанной ячейки (регистра) банка памяти.',
                        MEMORY_BANK: 'номер банка памяти (' + self.G_BYTE + ')',
                        MEMORY_LOCATION: 'адрес ячейки (регистр) памяти  (' + self.G_BYTE + ')'},
                    dali1.raw_write_memory.__name__: {
                        DESC: 'Запись значения в указанную ячейку (регистр) банка памяти',
                        MEMORY_BANK: 'номер банка памяти (' + self.G_BYTE + ')',
                        MEMORY_LOCATION: 'адрес ячейки (регистр) памяти (' + self.G_BYTE + ')',
                        MEMORY_VALUE: 'значение для записи в ячейку (регистр) памяти  (' + self.G_BYTE + ')'},
                    dali1.set_short_address.__name__: {
                        DESC: 'Установка устройству нового адреса',
                        SHORT_ADDRESS: 'новый адрес D0..63'},
                    dali1.swap_short_addresses.__name__: {
                        DESC: 'Замена адресов у двух устройств',
                        ADDRESS1: self.G_ADDRESS,
                        ADDRESS2: self.G_ADDRESS},
                    dali1.set_actual_level.__name__: {
                        DESC: 'Аналог команды IEC 62386-102, 11.3.1, DAPC (level)',
                        LEVEL: self.G_LEVEL},
                    dali1.query_actual_level.__name__: {
                        DESC: 'Запрашивает текущую яркость светильника'},
                    dali1.query_dimming_curve.__name__: {
                        DESC: 'Запрашивает значение установленной кривой диммирования'},
                    dali1.query_fade_time.__name__: {
                        DESC: 'Запрашивает время включения/выключения светильника'},
                    dali1.query_min_level.__name__: {
                        DESC: 'Запрашивает минимальный уровнень яркости светильника'},
                    dali1.query_scene_level.__name__: {
                        DESC: 'Запрашивает уровнень указанной сцены устройства',
                        SCENE_NUMBER: "Номер сцены 0..15 через пробел (по умолчанию - все сцены)"},
                    dali1.query_device_types.__name__: {
                        DESC: 'Запрашивает все типы, которые поддерживает устройство'},
                    dali1.query_fade_time_and_rate.__name__: {
                        DESC: 'Запрашивает время и скорость включения/выключения устройства'},
                    dali1.query_operating_mode.__name__: {
                        DESC: 'Запрашивает режим работы устройства'},
                    dali1.query_status.__name__: {
                        DESC: 'Запрашивает текущий статус устройства. В качестве ответа'},
                    dali1.query_groups.__name__: {
                        DESC: 'Запрашивает минимальный уровнень яркости светильника'},
                    dali1.query_physical_minimum.__name__: {
                        DESC: 'Запрашивает минимальный уровнень яркости светильника'},
                    dali1.query_system_failure_level.__name__: {
                        DESC: 'Запрашивает минимальный уровнень яркости светильника'},
                    dali1.query_fade_rate.__name__: {
                        DESC: 'Запрашивает минимальный уровнень яркости светильника'},
                    dali1.select_dimming_curve.__name__: {
                        DESC: 'Выбор кривой диммирования',
                        DIMMING_CURVE: '0-логарифмическая; 1-прямая;'},
                    dali1.identify.__name__: {
                        DESC: 'Включение режима идентификации',
                        TIME: 'время (в секундах) работы режима идентификации. По умолчанию 3600 секунд',
                        LEVEL: 'уровень, включаемый по завершении режима идентификации'},
                    dali1.discovery.__name__: {
                        DESC: 'Включение софтового режима идентификации',
                        TIME: 'время (в секундах) работы режима идентификации. По умолчанию 3600 секунд',
                        LEVEL: 'уровень, включаемый по завершении режима идентификации'},
                    dali1.add_to_group.__name__: {
                        DESC: 'Добавление устройства в группу',
                        GROUP_IDs: 'Идентификаторы групп 0..15 через пробел'},
                    dali1.remove_from_group.__name__: {
                        DESC: 'Удаление устройства из группы',
                        GROUP_IDs: 'Идентификаторы групп 0..15 через пробел'},
                    dali1.extend_line.__name__: {
                        DESC: self.G_EXTEND_LINE,
                        MODE: self.G_MODE,
                        START_ADDRESS: self.G_START_ADDRESS},
                    dali1.test_line.__name__: {
                        DESC: 'Статистика ответов на запрос яркости',
                        COUNT: 'Количество запросов для каждого устройства (по умолчанию - %s)' % dali1.test_line.DEF_COUNT,
                        COMMAND: 'Команда для выполнения (по умолчанию - %s)' % dali1.test_line.DEF_COMMAND.__name__,
                        ADDRESSES: 'Адреса устройств через запятую (по умолчанию - все)',
                        SORT_BY_ADDR: 'Сортировка по адресам (по умолчанию - по частоте ответов)'},
                    dali1.construct_line.__name__: {
                        DESC: self.G_CONSTRUCT_LINE,
                        MODE: self.G_MODE,
                        START_ADDRESS: self.G_START_ADDRESS},
                    dali1.fix_double_addresses.__name__: {
                        DESC: self.G_FIX_DOUBLE_ADDRESSES,
                        MODE: self.G_MODE,
                        START_ADDRESS: self.G_START_ADDRESS},
                    dali1.scan_line.__name__: {
                        DESC: self.G_SCAN_LINE},
                    dali1.dump.__name__: {
                        DESC: self.G_DUMP},
                },
                DALI2: {
                    dali2.raw_command.__name__: {
                        DESC: 'Выполнение "сырой" команды',
                        DEVICE_ADDR: self.G_BYTE,
                        INSTANCE: self.G_BYTE,
                        BYTE_OPCODE: self.G_BYTE,
                        SEND_TWICE: self.G_SEND_TWICE},
                    dali2.raw_command_dtr0.__name__: {
                        DESC: 'Выполнение "сырой" команды, с использованием DTR0',
                        BYTE_ADDR: self.G_BYTE,
                        BYTE_INSTANCE: self.G_BYTE,
                        BYTE_OPCODE: self.G_BYTE,
                        DTR0_VALUE: self.G_DTR0_VALUE,
                        SEND_TWICE: self.G_SEND_TWICE},
                    dali2.raw_query.__name__: {
                        DESC: 'Выполнение "сырого" запроса',
                        DEVICE_ADDR: self.G_BYTE,
                        INSTANCE: self.G_BYTE,
                        BYTE_OPCODE: self.G_BYTE,
                        SEND_TWICE: self.G_SEND_TWICE},
                    dali2.raw_read_memory.__name__: {
                        DESC: 'Чтение значения из памяти устройства по указанным области памяти и адресу ячейки',
                        MEMORY_BANK: 'Идентификатор банка памяти',
                        MEMORY_LOCATION: 'Адрес ячейки памяти'},
                    dali2.raw_write_memory.__name__: {
                        DESC: 'Запись значения в память устройства',
                        MEMORY_BANK: 'Идентификатор банка памяти',
                        MEMORY_LOCATION: 'Адрес ячейки памяти',
                        MEMORY_VALUE: 'Один байт в hex-виде'},
                    dali2.set_short_address.__name__: {
                        DESC: 'Установка устройству нового адреса',
                        SHORT_ADDRESS: 'новый адрес D0..63'},
                    dali2.set_operating_mode.__name__: {
                        DESC: 'Установка режима устройства',
                        OPERATING_MODE: 'устанавливаемое значение'},
                    dali2.start_quiescent_mode.__name__: {
                        DESC: 'Включить режим тишины'},
                    dali2.stop_quiescent_mode.__name__: {
                        DESC: 'Выключить режим тишины'},
                    dali2.enable_power_cycle_notification.__name__: {
                        DESC: 'Включить отправку событий питания'},
                    dali2.disable_power_cycle_notification.__name__: {
                        DESC: 'Выключить отправку событий питания'},
                    dali2.save_persistent_variables.__name__: {
                        DESC: 'Перезаписать энерго-независимую память'},
                    dali2.query_device_capabilities.__name__: {
                        DESC: '11.6.2, QUERY DEVICE CAPABILITIES'},
                    dali2.query_device_status.__name__: {
                        DESC: '11.6.3, QUERY DEVICE STATUS'},
                    dali2.query_number_of_instances.__name__: {
                        DESC: 'Количество типов, релизованных в устройстве'},
                    dali2.query_operating_mode.__name__: {
                        DESC: 'Запрос режма работы устройства'},
                    dali2.query_quiescent_mode.__name__: {
                        DESC: 'Включен ли режм тишины'},
                    dali2.query_power_cycle_notification.__name__: {
                        DESC: 'Включен ли режим уведомлений по событиям питания'},
                    dali2.enable_instance.__name__: {
                        DESC: 'Выключить датчик'},
                    dali2.disable_instance.__name__: {
                        DESC: 'Включить датчик'},
                    dali2.set_primary_instance_group.__name__: {
                        DESC: 'Установить целевую группу 0',
                        GROUP_ID: self.G_GROUP_ID_0_31},
                    dali2.set_instance_group_1.__name__: {
                        DESC: 'Установить целевую группу 1',
                        GROUP_ID: self.G_GROUP_ID_0_31},
                    dali2.set_instance_group_2.__name__: {
                        DESC: 'Установить целевую группу 2',
                        GROUP_ID: self.G_GROUP_ID_0_31},
                    dali2.set_event_scheme.__name__: {
                        DESC: 'Установить схему адресации событий',
                        EVENT_SCHEME: '0 - тип/индекс; 1 - адрес/тип; 2 - адрес/индекс; 3 - группа устройства/тип; 4 - группа типа/тип'},
                    dali2.set_event_priority.__name__: {
                        DESC: 'Установить приоритет отправки события',
                        EVENT_PRIORITY: '2..5'},
                    dali2.set_event_filter.__name__: {
                        DESC: 'Установить Фильтр типов отправляемых событий',
                        EVENT_FILTER: '0x0..0xFEFEFE'},
                    dali2.query_instance_type.__name__: {
                        DESC: 'Тип устройства'},
                    dali2.query_resolution.__name__: {
                        DESC: 'Разрешение устройства'},
                    dali2.query_instance_status.__name__: {
                        DESC: 'Статус устройства'},
                    dali2.query_instance_enabled.__name__: {
                        DESC: 'Включено ли устройство'},
                    dali2.query_primary_instance_group.__name__: {
                        DESC: 'Целевая группа 0'},
                    dali2.query_instance_group_1.__name__: {
                        DESC: 'Целевая группа 1'},
                    dali2.query_instance_group_2.__name__: {
                        DESC: 'Целевая группа 2'},
                    dali2.query_event_scheme.__name__: {
                        DESC: 'Схема адресации событий'},
                    dali2.query_event_priority.__name__: {
                        DESC: 'Приоритет отправки события'},
                    dali2.remove_short_address.__name__: {
                        DESC: 'Удаляет короткий адрес по указанному адресу'},
                    dali2.add_to_group.__name__: {
                        DESC: 'Добавляет устройство в указанную группу',
                        GROUP_IDs: 'Идентификаторы групп 0..31 через пробел'},
                    dali2.remove_from_group.__name__: {
                        DESC: 'Удаление устройства из группы',
                        GROUP_IDs: 'Идентификаторы групп 0..31 через пробел'},
                    dali2.query_device_groups.__name__: {
                        DESC: 'Запрашивает все группы, в которые входит устройство'},
                    dali2.query_feature_types.__name__: {
                        DESC: 'Запрашивает все типы расширений, которые поддерживает компонент (instance) устройства'},
                    dali2.query_instance_input.__name__: {
                        DESC: 'Запрашивает у компонента (instance) устройства значение его входа (сенсора)'},
                    dali2.query_event_filter.__name__: {
                        DESC: 'Запрашивает у компонента (instance) устройства набор включенных событий'},
                    dali2.read_firmware_version.__name__: {
                        DESC: 'Запрашивает версию прошивки устройства'},
                    dali2.read_hardware_version.__name__: {
                        DESC: 'Запрашивает аппаратную версию устройства'},
                    dali2.read_gtin.__name__: {
                        DESC: 'Запрашивает GTIN устройства'},
                    dali2.read_gtin_oem.__name__: {
                        DESC: 'Запрашивает OEM GTIN устройства'},
                    dali2.read_serial.__name__: {
                        DESC: 'Запрашивает серийный номер (identification number) устройства'},
                    dali2.read_serial_oem.__name__: {
                        DESC: 'Запрашивает OEM серийный номер (OEM identification number) устройства'},
                    dali2.identify.__name__: {
                        DESC: 'Включает режим идентификации устройства',
                        TIMEOUT: 'время работы режимана (по умолчанию 1 час)'},
                    dali2.swap_short_addresses.__name__: {
                        DESC: 'Обменивает короткие адреса у двух указанных устройств',
                        ADDRESS1: self.G_ADDRESS,
                        ADDRESS2: self.G_ADDRESS},
                    dali2.scan_line.__name__: {
                        DESC: self.G_SCAN_LINE},
                    dali2.test_line.__name__: {
                        DESC: 'Статистика ответов на запрос яркости',
                        COUNT: 'Количество запросов для каждого устройства (по умолчанию - %s)' % dali2.test_line.DEF_COUNT,
                        COMMAND: 'Команда для выполнения (по умолчанию - %s)' % dali2.test_line.DEF_COMMAND.__name__,
                        ADDRESSES: 'Адреса устройств через запятую (по умолчанию - все)',
                        SORT_BY_ADDR: 'Сортировка по адресам (по умолчанию - по частоте ответов)'},
                    dali2.extend_line.__name__: {
                        DESC: self.G_EXTEND_LINE,
                        MODE: self.G_MODE,
                        START_ADDRESS: self.G_START_ADDRESS},
                    dali2.construct_line.__name__: {
                        DESC: self.G_CONSTRUCT_LINE,
                        MODE: self.G_MODE,
                        START_ADDRESS: self.G_START_ADDRESS},
                    dali2.fix_double_addresses.__name__: {
                        DESC: self.G_FIX_DOUBLE_ADDRESSES,
                        MODE: self.G_MODE,
                        START_ADDRESS: self.G_START_ADDRESS},
                    dali2.dump.__name__: {
                        DESC: self.G_DUMP}
                },
                GATEWAY: {
                    gateway.status.__name__: {
                        DESC: 'Статус удалённых шлюзов',
                        GATEWAY_IDs: 'id шлюзов через пробел (по умолчанию - все шлюзы)'},
                    gateway.connect.__name__: {
                        DESC: 'Создает и соединяет локальный виртуальный CAN-интерфейс (rcan<InterfaceID>) с '
                              'физическим CAN-интерфейсом удаленного (адрес <GatewayIP>) шлюза',
                        IP: self.G_IP_TYPE,
                        PORT: 'локальный порт, который будет слушать эмулятор шлюза',
                        ATTRIBUTES: 'параметры файла конфигурации'},
                    gateway.disconnect.__name__: {
                        DESC: 'Отсоединяет локальный виртуальный CAN-интерфейс (rcan<InterfaceID>) от '
                              'физического CAN-интерфейса удаленного шлюза и удаляет локальный интерфейс'},
                    gateway.up.__name__: {
                        DESC: 'Запускает сервис, транслирующий сообщения шлюза'},
                    gateway.down.__name__: {
                        DESC: 'Останавливет сервис, транслирующий сообщения шлюза'},
                },
                BRIDGE: {
                    bridge.start.__name__: {
                        DESC: 'Комплексный запуск локального эмулятора шлюза с созданием конфигураций '
                              'запуска/автозапуска и привязкой его к локальному физическому CAN-интерфейсу (can0)',
                        PORT: 'локальный порт, используемый эмулятором шлюза',
                        ATTRIBUTES: 'параметры файла конфигурации'},
                    bridge.stop.__name__: {
                        DESC: 'Комплексное удаление локального эмулятора шлюза'},
                    bridge.up.__name__: {
                        DESC: 'Запускает сервис эмуляции шлюза'},
                    bridge.down.__name__: {
                        DESC: 'Останавливет сервис эмуляции шлюза'},
                    bridge.status.__name__: {
                        DESC: 'Статус локального эмулятора шлюза'},
                },
            }
        return self._loc


loc = Loc()
