import can


CONTROLLER_ID = 31


class CanConnection:
    def __init__(self, interface):
        self.bus = can.interface.Bus(channel=interface, **dict(bustype='socketcan'))

    def set_filter(self, module_address, forward=False):
        f = [{
                "can_id": (module_address << 5) + CONTROLLER_ID,
                "can_mask": 1023,
            }]

        if forward:
            f.append({
                "can_id": (CONTROLLER_ID << 5) + module_address,
                "can_mask": 1023,
            })

        self.bus.set_filters(f)

    def stop(self):
        self.bus.shutdown()

    @staticmethod
    def check_can(interface: str, timeout=3):
        message = can.Message(arbitration_id=2080, data=bytearray([1, 2, 3]), extended_id=False)
        try:
            bus = can.interface.Bus(channel=interface, **dict(bustype='socketcan'))
            bus.send(message, timeout)
        except can.CanError:
            return False
        else:
            return True
