from resh.utils.logger import log
from .can_packet import CanSend, CanDump
from .rapida import commands as rcmds
from .rapida import responses as rresp
from ..payloads.rapida import ServicePacket


class RapidaSend(CanSend):
    def __init__(self, module_type, module_address, end_frame, **kwargs):
        super(RapidaSend, self).__init__(module_type, module_address, end_frame, cmd=1, **kwargs)

    def check_commit(self, dump):
        pass

    def on_response(self, dump):
        if isinstance(dump, RapidaDump):
            log.info('   <- %s' % dump.payload.command)
            if (dump.command_byte == self.end_frame[0] + 1
                    or (dump.command_byte == rresp.FirmwareVersion._opcode
                        and self.end_frame[0] == rcmds.Reboot._opcode
                    )
            ):
                self.stop_answer_timer()
                return dump.payload.command

    @property
    def committed(self):
        return True

    @property
    def frame(self):
        return ((1 << 7) + self.module_type).to_bytes(1, 'big') + self.end_frame


class RapidaDump(CanDump):
    cmd = 1

    @property
    def command_byte(self):
        return self.frame[1]

    @property
    def data(self):
        return self.frame[2:]

    @property
    def payload(self):
        rs = rresp.of(self.command_byte, self.data)
        if rs.__class__ == rresp.Response:
            cm = rcmds.of(self.command_byte, self.data)
            if cm.__class__ != rcmds.Command and cm.__class__ != rcmds.Query:
                return ServicePacket(self, cm)
        return ServicePacket(self, rs)
