from . import gear


class FeatureStatusResponse(gear.command.BitmapResponse):
    bits = ["Automatic Activation"]


class ColourStatusResponse(gear.command.BitmapResponse):
    bits = [
        "xy-coordinate colour point out of range",
        "Colour temperature TC out of range",
        "Auto calibration running",
        "Auto calibration successful",
        "Colour type xy-coordinate active",
        "Colour type colour temperature TC active",
        "Colour type primary N active",
        "Colour type RGBWAF active"
    ]


class BitmapOfRanges(gear.command.BitmapResponse):
    ranges = None

    @property
    def status(self):
        res = super(BitmapOfRanges, self).status
        for r in self.ranges:
            value = self._value[r[0]:r[1]]

            if isinstance(self.ranges[r], (list, tuple)):
                value = self.ranges[r][1][value]
                res.append(f'{self.ranges[r][0]} is {value}')
            else:
                res.append(f'{self.ranges[r]} is {value}')

        return res


class ColourTypeFeaturesResponse(BitmapOfRanges):
    bits = ['xy-coordinate capable', 'Colour temperature TC capable']
    ranges = {(2, 4): 'Number of primaries',
              (5, 7): 'Number of RGBWAF channels'}


class QueryRgbwafControlResponse(BitmapOfRanges):
    bits = [
               'Red linked',
               'Green linked',
               'Blue linked',
               'White linked',
               'Amber linked',
               'Freecolor linked',
           ]

    ranges = {
        (6, 7): ('control type', ['Channel', 'Colour', 'Normalised'])
    }


class QueryAssignedColourResponse(gear.command.BitmapResponse):
    bits = [
        'not assigned',
        'red',
        'Green',
        'Blue',
        'White',
        'Amber',
        'Freecolor',
    ]

    @property
    def status(self):
        if self._value == 255:
            return 'MASK'

        return super(QueryAssignedColourResponse, self).status
