from resh.utils.errors import UnavailableMethod


def add_metaclass(metaclass):
    """Class decorator for creating a class with a metaclass."""

    def wrapper(cls):
        orig_vars = cls.__dict__.copy()
        orig_vars.pop('__dict__', None)
        orig_vars.pop('__weakref__', None)
        slots = orig_vars.get('__slots__')
        if slots is not None:
            if isinstance(slots, str):
                slots = [slots]
            for slots_var in slots:
                orig_vars.pop(slots_var)
        return metaclass(cls.__name__, cls.__bases__, orig_vars)
    return wrapper


class DomainTracker(type):
    """Metaclass keeping track of all the types of Domain we understand.

    Domains that have names starting with '_' are treated as abstract
    base classes that will never be instantiated.
    """

    def __init__(cls, name, bases, attrs):
        super(DomainTracker, cls).__init__(name, bases, attrs)
        if not hasattr(cls, '_commands'):
            cls._commands = []
        else:
            if cls.__name__[0] != '_':
                cls._commands.append(cls)

    @property
    def commands(self): return self._commands

    def from_name(self, name):
        for dc in self._commands:
            if dc.__name__ == name:
                return dc

        raise UnavailableMethod(name)
