import sys


class Completer(object):  # Custom completer
    @staticmethod
    def start(entries):
        completer = Completer(entries)
        if sys.platform == 'linux':
            import readline
            readline.set_completer(completer.complete)
            readline.parse_and_bind('tab: complete')
            return completer
        else:
            print('Completer not available')

        return None

    def __init__(self, entries):
        self.entries = entries

    def build_scope(self, str_scope: str):
        scope = str_scope.split()
        options = self.entries

        while len(scope) > 0:
            part = scope.pop(0)
            if part in options:
                options = options[part]

        if isinstance(options, dict):
            options = list(options.keys())
        return options

    def complete(self, text, state):
        if sys.platform == 'linux':
            import readline
            beg = readline.get_begidx()
            line = readline.get_line_buffer()[:beg]
            options = self.build_scope(line)

            if state == 0:  # on first trigger, build possible matches
                if text:  # cache matches (entries that start with entered text)
                    self.matches = [s for s in options
                                    if s and s.startswith(text)]
                else:  # no text entered, all matches possible
                    self.matches = options[:]

            try:
                return self.matches[state] + ' '
            except IndexError:
                return None
