import os

import resh

template = """
resh_completion() {
    declare -A _domains
    #INITIALIZATION
    
    COMPREPLY=()
    cur="${COMP_WORDS[COMP_CWORD]}"
    
    if [[ ${COMP_CWORD} == 0 ]] ; then
        compopt -o default
    elif [[ ${COMP_CWORD} == 1 ]] ; then
        mapfile -t COMPREPLY < <(compgen -W "${_main}" -- "${cur}")
    elif [[ ${COMP_CWORD} == 2 ]] && [[ ${COMP_WORDS[1]} == "help" ]] ; then
        mapfile -t COMPREPLY < <(compgen -W "${_help}" -- "${cur}")
    else
        _subcmd=${COMP_WORDS[$COMP_CWORD-1]}
        if [[ ${_subcmd} != "--file" ]] && [[ ${COMP_WORDS[$COMP_CWORD-2]} != "update_firmware" ]] && [[ ${COMP_WORDS[$COMP_CWORD-1]} != "update_power_firmware" ]] && [[ ${_subcmd} != ">" ]] ; then
            mapfile -t COMPREPLY < <(compgen -W "${_domains[${COMP_WORDS[$COMP_CWORD-1]}]}" -- "${cur}")
        else
            compopt -o default
        fi
    fi
}

complete -F resh_completion resh
"""


def build(dest=None):
    _domains = {domain: [cmd.__name__ for cmd in tracker.commands] for domain, tracker in resh.DOMAINS.items()}
    _help = list(_domains.keys())
    _main = f'_main="{" ".join(_help + ["help", "version"])}"'
    _help = f'_help="{" ".join(_help)}"'
    _domains = '\n    '.join(f'_domains[{k}]="{" ".join(v)}"' for k, v in _domains.items())

    s = '\n    '.join([_main, _help, _domains])

    s = template.replace('#INITIALIZATION', s)
    if dest is None:
        print(s)
    else:
        os.makedirs(os.path.dirname(dest), exist_ok=True)

        try:
            with open(dest, 'r') as fs:
                for i in s.splitlines(keepends=True):
                    assert fs.readline() == i
                assert fs.readline() == ''
        except (FileNotFoundError, FileExistsError, AssertionError, EOFError):
            with open(dest, 'w') as fs:
                fs.write(s)
