from resh.protocols.payloads import Payload
from resh.protocols.payloads.dali import DaliCommand, Dali2Command, Dali2Event, DaliPayload, DaliResponse
from resh.protocols.payloads.rapida import ServicePacket
from resh.utils import tools
from resh.utils.const import DALI, DALI2, EVENT, SYS


POSITIVE = 0
NEGATIVE = 1

    
class Filter:
    _domains = None
    _modules = None
    _channels = None
    _addresses = None
    _instances = None
    _names = None

    def __init__(self, filters: list):
        self._parse(list(filters))

    def _parse(self, filters):
        try:
            self._modules = self._add_to_stack(self._modules, filters.pop(0), tools.parse_module)
            self._channels = self._add_to_stack(self._channels, filters.pop(0), tools.parse_channel)
            self._addresses = self._add_to_stack(self._addresses, filters.pop(0).upper(), tools.parse_address, False)
            self._instances = self._add_to_stack(self._instances, filters.pop(0).upper(), tools.parse_instance, False)
            self._domains = self._add_to_stack(self._domains, filters.pop(0).lower(), tools.parse_domain)
            self._names = self._add_to_stack(self._names, filters.pop(0))

        except IndexError:
            return

    @staticmethod
    def _add_to_stack(owner, values, parser=None, use_parser=True):
        if isinstance(values, str):
            if values == '0':
                return owner
            values = values.split(',')

        for item in values:
            if item.startswith('-'):
                item = item[1:]
                index = NEGATIVE
            else:
                index = POSITIVE

            if parser:
                _item = parser(item)

                if use_parser:
                    item = _item

            if owner is None:
                owner = [[], []]
            owner[index].append(item)

        return owner

    @staticmethod
    def _check_stack(stack, value):
        if stack[NEGATIVE] and value in stack[NEGATIVE]:
            return False
        elif stack[POSITIVE] and value not in stack[POSITIVE]:
            return False

        return True
    
    def check(self, payload: Payload) -> bool:
        """Print if True"""
        if isinstance(payload, Dali2Event):
            pass
        if self.check_domain(payload):
            if self.check_module(payload):
                if self.check_channel(payload):
                    if self.check_address(payload):
                        if self.check_instance(payload):
                            if self.check_name(payload):
                                return True

        return False

    def check_domain(self, payload: Payload) -> bool:
        if self._domains:
            if isinstance(payload, ServicePacket):
                return self._check_stack(self._domains, SYS)
            elif isinstance(payload, Dali2Event):
                return self._check_stack(self._domains, EVENT)
            elif isinstance(payload, Dali2Command) or (isinstance(payload, DaliResponse) and payload.is_dali2):
                return self._check_stack(self._domains, DALI2)
            elif isinstance(payload, DaliCommand) or (isinstance(payload, DaliResponse) and not payload.is_dali2):
                return self._check_stack(self._domains, DALI)
            else:
                return False

        return True

    def check_module(self, payload: Payload) -> bool:
        if self._modules:
            return self._check_stack(self._modules, payload.dump.module_address)
        return True

    def check_channel(self, payload: Payload) -> bool:
        if self._channels:
            if isinstance(payload, ServicePacket) and hasattr(payload, 'channel'):
                _channel = payload.command.channel
            else:
                _channel = payload.dump.channel

            return self._check_stack(self._channels, _channel)
        return True

    def check_address(self, payload: Payload) -> bool:
        if self._addresses and isinstance(payload, DaliPayload):
            return payload.has_address and self._check_stack(self._addresses, payload.address)
        return True

    def check_instance(self, payload: Payload) -> bool:
        if self._instances and isinstance(payload, DaliPayload) and payload.has_instance:
            return self._check_stack(self._instances, payload.instance)
        return True

    def check_name(self, payload: Payload) -> bool:
        if self._names:
            return self._check_stack(self._names, payload.command.__class__.__name__)
        return True
