import os
import signal

from . import logger

HELP = 'help'
VERSION = 'version'
LOG_LEVEL = '-l'
DEBUG = '-d'
JSON = '--json'
VERBOSE = '-v'
FILE = '--file'
INTERACTIVE = 'interactive_mode'
COMPLETION_MODE = '--generate_completions'
INTERFACE = '--gateway'


class Settings(object):
    def __init__(self):
        self.interactive_mode = False
        self.help = False
        self.version = False
        self.debug = False
        self.file = None
        self.completion_mode = False
        self._interface = 'can0'
        self.quiescent = False

    def parse(self, args: list) -> list:
        if DEBUG in args:
            if not self.debug:
                _index = args.index(DEBUG)
                args.pop(_index)
                if len(args) > _index and '-' not in args[_index]:
                    _value = args.pop(_index)
                    if ':' in _value and str(_value.split(':')[1]).isdigit():
                        _value = _value.split(':')
                        _value[1] = int(_value[1])
                    else:
                        raise BaseException('"{}" parameter required as "0.0.0.0:0000", but yet as "{}"'.format(DEBUG, _value))
                else:
                    _value = ('10.10.1.109', 54105)
                print('Wait for debug attach from {} ...'.format(':'.join(str(i) for i in _value)))

                def suicide(signum, frame):
                    pid = os.popen('ps | grep resh').read().strip().split()[0]
                    os.system(f'kill -9 {pid}')

                signal.signal(signal.SIGALRM, suicide)
                signal.alarm(3)
                import pydevd_pycharm
                pydevd_pycharm.settrace(_value[0], port=_value[1], stdoutToServer=True, stderrToServer=True)

                signal.alarm(0)
                
                self.debug = _value
        else:
            self.debug = None

        if HELP in args:
            self.help = True
            args.remove(HELP)
        else:
            self.help = False

        if VERSION in args:
            self.version = True
            args.remove(VERSION)
        else:
            self.version = False

        if JSON in args:
            logger.log.set_json_out(True)
            args.remove(JSON)

        if VERBOSE in args:
            self.quiescent = True
            args.remove(VERBOSE)

        if COMPLETION_MODE in args:
            self.completion_mode = True
            args.remove(COMPLETION_MODE)
        else:
            self.completion_mode = False

        if FILE in args:
            _index = args.index(FILE)
            _q = args.pop(_index)
            if len(args) > _index:
                _value = args.pop(_index)

                if os.path.isfile(_value):
                    self.file = _value
                else:
                    raise BaseException(f'"{_value}" is not a file')
            else:
                raise BaseException(f'"{FILE}" assume file path as string')
        else:
            self.file = None

        if INTERFACE in args:
            _index = args.index(INTERFACE)
            _ = args.pop(_index)
            if len(args) > _index:
                _interface = args.pop(_index)

                self.interface = _interface
            else:
                raise BaseException(f'"{INTERFACE}" assume gateway id (1 for "rcan1"")')

        if LOG_LEVEL in args:
            _index = args.index(LOG_LEVEL)
            args.pop(_index)
            if len(args) > _index and '-' not in args[_index]:
                _value = args[_index]
                if str(_value).isdigit():
                    _value = int(_value)
                else:
                    raise BaseException('"{}" parameter required as <int>, but yet as "{}"'.format(LOG_LEVEL, _value))
            else:
                _value = 1
            logger.log.set_log_level(_value)
        else:
            logger.log.set_log_level(0)

        self.interactive_mode = args[0] == INTERACTIVE

        return args

    @property
    def interface(self): return self._interface

    @interface.setter
    def interface(self, value):
        if str(value).isdigit():
            if str(value) == '0':
                self._interface = f'can0'
            else:
                self._interface = f'rcan{value}'
        else:
            self._interface = value


conf = Settings()
